# REVOLUTION_MOD
#
# by jdog5000
# Version 0.72


from CvPythonExtensions import *
import CvUtil
import PyHelpers		
import Popup as PyPopup
import pickle
import RevUtils

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
game = CyGame()

class Revolution :

    def __init__(self, customEM, config):

        CvUtil.pyPrint("Initializing Revolution Mod")

        ####### System Variables #######
        self.EventKeyDown=6

        self.pickCivToSplitPopup = 7004
        self.revolutionPopup = 7007
        self.joinHumanPopup = 7009
        self.controlLostPopup = 7011

        ####### Revolution Variables ##########

        self.config = config
        self.customEM = customEM

        self.LOG_DEBUG = config.getboolean("Revolution", "DebugMode", False)
        self.enableRevoltStatusPopup = config.getboolean("Revolution", "EnableRevoltStatusPopup", True)
        self.allowAssimilation = config.getboolean("Revolution", "AllowAssimilation", True)
        self.revsGetFirstMove = config.getboolean("Revolution", "RevGetFirstMove", False)
        self.allowSmallBarbRevs = config.getboolean("Revolution", "AllowSmallBarbRevs", True)

        # Fewer than this number of cities won't try for independence, but change leader
        #self.minCitiesForIndependence = config.getint("Revolution","MinCitiesForIndependence",4)
        # Cities considered close to revolution instigator inside this radius
        self.closeRadius = config.getint("Revolution","CloseRadius",10)

        #### Cities may attempt to leave and join dominant culture
        self.culturalRevolution = config.getboolean("Revolution", "CulturalRevolution", True)
        # Cities with at least this level of your nationality won't start cultural revolutions
        self.maxNationalityThreshold = config.getint("Revolution", "MaxNationality", 40)

        #### Cities may ask for religious change or independence for religious reasons
        self.religiousRevolution = config.getboolean("Revolution", "ReligiousRevolution", True)
        # Cities with your state religion can join a religious revolution
        self.allowStateReligionToJoin = config.getboolean("Revolution", "AllowStateReligionToJoin", True)

        #### Cities may ask for civics changes
        self.civicRevolution = config.getboolean("Revolution", "CivicRevolution", True)

        #### Cities may ask/demand changes in leadership
        self.leaderRevolution = config.getboolean("Revolution", "LeaderRevolution", True)

        #### Cities may ask/demand independence
        # This is the base kind of revolution, not really negotiable if you have this component on
        self.independenceRevolution = True

        #self.warningThreshold = config.getint("Revolution", "HumanWarningThreshold", 900)
        self.warnFrac = config.getfloat("Revolution", "HumanWarnFrac", .9)
        self.warnTurns = config.getint("Revolution", "WarnTurns", 15)
        #self.revReadyThreshold = config.getint("Revolution", "JoinRevolutionThreshold", 600)
        self.revReadyFrac = config.getfloat("Revolution", "JoinRevolutionFrac", .6)
        self.revInstigatorThreshold = config.getint("Revolution", "InstigateRevolutionThreshold", 1000)
        self.alwaysViolentThreshold = config.getint("Revolution", "AlwaysViolentThreshold", 1700)
        self.badLocalThreshold = config.getint("Revolution", "BadLocalThreshold", 8)
        self.turnsBetweenRevs = config.getint("Revolution", "TurnsBetweenRevs", 8)
        self.deniedTurns = config.getint("Revolution", "DeniedTurns", 5)
        self.acceptedTurns = config.getint("Revolution", "AcceptedTurns", 10)
        self.acquiredTurns = config.getint("Revolution", "AcquiredTurns", 10)
        self.buyoffTurns = config.getint("Revolution", "BuyoffTurns", 10)

        self.warWearinessMod = config.getfloat("Revolution", "WarWearinessMod", 1.0)

        # Change chances a revolution occurs given conditions
        self.revolutionModifier = config.getfloat("Revolution", "ChanceModifier", 1.0)
        # Change strength of revolutions that resort to violence
        self.strengthModifier = config.getfloat("Revolution", "StrengthModifier", 1.0)
        # Change strength of city area revolution factor
        self.colonyModifier = config.getfloat("Revolution", "ColonyModifier", 1.0)
        # Change strength of civ size revolution factor
        self.civSizeModifier = config.getfloat("Revolution", "CivSizeModifier", 1.0)
        # Change strength of religion revolution factor
        self.religionModifier = config.getfloat("Revolution", "ReligionModifier", 1.0)
        # Change strength of losing a city on revolution indices
        self.cityLostModifier = config.getfloat("Revolution", "CityLostModifier", 1.0)
        # Change strength of gaining a city on revolution indices
        self.cityAcquiredModifier = config.getfloat("Revolution", "CityAcquiredModifier", 1.0)
        # Change the initial strength of rev culture in a captured city (1.0 = 50% nationality)
        self.revCultureModifier = config.getfloat("Revolution", "RevCultureModifier", 1.0)

        self.iRevRoll = 20000

        # Stores human leader type when human loses control of civ
        self.humanLeaderType = None

        ######################
        # Rest are initialized on firt end of game turn
        self.iNationalismTech = None
        self.iNationalismTech = None
        self.iLiberalismTech = None
        self.iSciMethodTech = None

        self.iCivicOptionReligion = None
        self.iPaganismID = None
        self.iOrganizedReligionID = None
        self.iTheocracyID = None
        self.iFreeReligionID = None
        self.iPacifismID = None

        self.iCivicOptionGovernment = None
        self.iDespotismID = None
        self.iHereditaryRuleID = None
        self.iRepresentationID = None
        self.iPoliceStateID = None
        self.iUniversalSuffrageID = None

        self.iCivicOptionLabor = None
        self.iTribalismID = None
        self.iSlaveryID = None
        self.iSerfdomID = None
        self.iCasteSystemID = None
        self.iEmancipationID = None

        self.iCivicOptionLegal = None		
        self.iBarbarismID = None
        self.iVassalageID = None
        self.iBureaucracyID = None
        self.iNationhoodID = None
        self.iFreeSpeechID = None

        self.iCivicOptionEconomy = None
        self.iDecentralizationID = None
        self.iMercantilismID = None
        self.iFreeMarketID = None
        self.iStatePropertyID = None
        self.iEnvironmentalismID = None

        ############ Holders for popup variables ##############

        self.cityList = None
        self.pPlayer = None
        self.revType = None
        self.peaceful = None
        self.pRevPlayer = None
        self.isJoinWar = None
        self.joinPlayer = None
        self.newLeaderType = None
        self.newLeaderName = None
        self.isElection = None
        self.newCivic = None
        self.newReligion = None
        self.iHappiness = None
        self.odds = None
        self.buyOffCost = None
        
        ############# Register events and popups ##############
        # City and civ events
        customEM.addEventHandler( "cityLost", self.onCityLost )
        #customEM.addEventHandler( "cityBuilt", self.onCityBuilt )
        customEM.addEventHandler( "cityAcquired", self.onCityAcquired )
        customEM.addEventHandler( "religionFounded", self.onReligionFounded )

        # All revolution index update and processing
        customEM.addEventHandler( "EndGameTurn", self.onEndGameTurn )
        customEM.addEventHandler( "BeginPlayerTurn", self.updatePlayerRevolution )
            
        # Popup launching and handling events
        if( self.enableRevoltStatusPopup ) :
            customEM.addEventHandler( "kbdEvent", self.onKbdEvent )
        self.customEM.setPopupHandler( self.revolutionPopup, ["revolutionPopup",self.revolutionPopupHandler,self.blankHandler] )
        self.customEM.setPopupHandler( self.joinHumanPopup, ["joinHumanPopup",self.joinHumanHandler,self.blankHandler] )
        self.customEM.setPopupHandler( self.controlLostPopup, ["controlLostPopup",self.controlLostHandler,self.blankHandler] )


    def blankHandler( self, playerID, netUserData, popupReturn ) :
        # Dummy handler to take the second event for popup
        return

    def showHumanPopup( self, playerID ) :
        # Determines whether to show popup to active player

        if( playerID == game.getActivePlayer() ) :
            if( self.revsGetFirstMove ) :
                if( game.getAIAutoPlay() <= 1 ) :
                    return True
            else :
                if( game.getAIAutoPlay() <= 1 ) :
                    return True

        return False

######################################################################

    def loadInfo( self ) :
        'on load or start'
        self.iNationalismTech = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_NATIONALISM')
        self.iLiberalismTech = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_LIBERALISM')
        self.iSciMethodTech = CvUtil.findInfoTypeNum(gc.getTechInfo,gc.getNumTechInfos(),'TECH_SCIENTIFIC_METHOD')

        self.iCivicOptionReligion = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo, gc.getNumCivicOptionInfos(), 'CIVICOPTION_RELIGION')
        self.iPaganismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_PAGANISM')
        self.iOrganizedReligionID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_ORGANIZED_RELIGION')
        self.iTheocracyID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_THEOCRACY')
        self.iFreeReligionID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_FREE_RELIGION')
        self.iPacifismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_PACIFISM')

        self.iCivicOptionGovernment = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo, gc.getNumCivicOptionInfos(), 'CIVICOPTION_GOVERNMENT')
        self.iDespotismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_DESPOTISM')
        self.iHereditaryRuleID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_HEREDITARY_RULE')
        self.iRepresentationID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_REPRESENTATION')
        self.iPoliceStateID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_POLICE_STATE')
        self.iUniversalSuffrageID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_UNIVERSAL_SUFFRAGE')

        self.iCivicOptionLabor = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo, gc.getNumCivicOptionInfos(), 'CIVICOPTION_LABOR')
        self.iTribalismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_TRIBALISM')
        self.iSlaveryID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_SLAVERY')
        self.iSerfdomID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_SERFDOM')
        self.iCasteSystemID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_CASTE_SYSTEM')
        self.iEmancipationID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_EMANCIPATION')

        self.iCivicOptionLegal = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo, gc.getNumCivicOptionInfos(), 'CIVICOPTION_LEGAL')		
        self.iBarbarismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_BARBARISM')
        self.iVassalageID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_VASSALAGE')
        self.iBureaucracyID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_BUREAUCRACY')
        self.iNationhoodID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_NATIONHOOD')
        self.iFreeSpeechID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_FREE_SPEECH')

        self.iCivicOptionEconomy = CvUtil.findInfoTypeNum(gc.getCivicOptionInfo, gc.getNumCivicOptionInfos(), 'CIVICOPTION_ECONOMY')
        self.iDecentralizationID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_DECENTRALIZATION')
        self.iMercantilismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_MERCANTILISM')
        self.iFreeMarketID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_FREE_MARKET')
        self.iStatePropertyID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_STATE_PROPERTY')
        self.iEnvironmentalismID = CvUtil.findInfoTypeNum(gc.getCivicInfo, gc.getNumCivicInfos(), 'CIVIC_ENVIRONMENTALISM')

    def onCityLost(self, argsList):
        'City Lost'
        city = argsList[0]
        player = gc.getPlayer(city.getOwner())

        self.playerCityLost( player, city, bConquest = (city.plot().getNumDefenders(player.getID()) == 0) )

    def onReligionFounded(self, argsList):
        'Religion Founded'
        iReligion, iFounder = argsList
        pPlayer = gc.getPlayer(iFounder)

        if( pPlayer.getStateReligion() >= 0 ) :
            if( not (pPlayer.getStateReligion() == iReligion) and not pPlayer.isAnarchy() ) :
                pCity = gc.getGame().getHolyCity(iReligion)
                curRevIdx = RevUtils.getCityRevolutionIndex( pCity )
                RevUtils.setCityRevolutionIndex( pCity, max([350,curRevIdx+100]) )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s founded non-state religion, index now %d ... state %d, new %d"%(pCity.getName(),RevUtils.getCityRevolutionIndex( pCity ),pPlayer.getStateReligion(),iReligion))

    def onEndGameTurn( self, argsList ) :

        if( self.iNationalismTech == None ) :
            self.loadInfo()

        self.topCivAdjustments( )

        if( self.allowAssimilation ) :
            self.checkForAssimilation( )

        self.checkReturnHumanLeader( )

    def updatePlayerRevolution( self, argsList ):

        iGameTurn, iPlayer = argsList

        if( iPlayer == gc.getBARBARIAN_PLAYER() ) :
            return

        if( self.iNationalismTech == None ) :
            self.loadInfo()
            
        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 and iPlayer == 0 ) : CvUtil.pyPrint("  Revolt - Rev index report for year %d"%(game.getGameTurnYear()))

        self.updateRevolutionCounters( iPlayer )
        self.updateLocalRevIndices( iPlayer )
        self.updateCivRevIndices( iPlayer )
        self.checkForRevolution( iPlayer )

        return

    def onKbdEvent(self, argsList ):
        'keypress handler'
        eventType,key,mx,my,px,py = argsList

        if ( eventType == self.EventKeyDown ):
	    theKey=int(key)
	    
	    if( theKey == int(InputTypes.KB_W) and self.customEM.bShift and self.customEM.bCtrl ) :
                playerPy = PyPlayer( game.getActivePlayer() )
                cityList = playerPy.getCityList()
                danger = 'Cities that might start a revolution\n'
                warning = '\nCities that might join a revolution\n'
                safe = '\nCities that will not revolt\n'
                for city in cityList :
                    pCity = city.GetCy()
                    revIdx = RevUtils.getCityRevolutionIndex( pCity )
                    if( revIdx >= self.revInstigatorThreshold ) :
                        danger += "%s\n"%(pCity.getName())
                    elif( revIdx >= int(self.revReadyFrac*self.revInstigatorThreshold) ) :
                        warning += "%s\n"%(pCity.getName())
                    else :
                        safe += "%s\n"%(pCity.getName())

                popup = PyPopup.PyPopup()
                popup.setHeaderString( 'Revolution Watch' )
                popup.setBodyString( danger + warning + safe )
                popup.launch()

######################################################################

    def playerCityLost( self, player, city, bConquest = True ) :

        if( player.getNumCities() < 1 or player.isBarbarian() ) :
            return

        iPlayer = player.getID()
        capital = player.getCapitalCity()
        capitalArea = capital.area().getID()

        iRevIdxChange = 0

        if( game.getGameTurn() - city.getGameTurnAcquired() < 2 ) :
            iRevIdxChange = 0
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: very recently acquired, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( game.getGameTurn() - city.getGameTurnAcquired() < 20 ) :
            iRevIdxChange = 50
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: recently acquired, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.isCapital() ) :
            iRevIdxChange = 400
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: capital, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.getHighestPopulation() < 4 ) :
            iRevIdxChange = 50
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: small city, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.plot().calculateCulturePercent(iPlayer) > 60 and city.countTotalCultureTimes100() > 100*100 ) :
            iRevIdxChange = 150
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: majority nationality, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        elif( city.getOriginalOwner() == iPlayer ) :
            iRevIdxChange = 125
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: original founder, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
        else :
            iRevIdxChange = 100
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s: default, %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),iRevIdxChange))
            
        if( not city.area().getID() == capitalArea and iRevIdxChange > 25 ) :
            iRevIdxChange -= 25
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is a colony, reducing effect to %d"%(city.getName(),iRevIdxChange))

        if( not bConquest ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Not conquest, reducing rev index increase from %d"%(iRevIdxChange))
            iRevIdxChange = iRevIdxChange/3.0

        iRevIdxChange = int( self.cityLostModifier*iRevIdxChange )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Loss of %s by %s (%d bConq): %d rev idx change"%(city.getName(),player.getCivilizationDescription(0),bConquest,iRevIdxChange))

        for city in PyPlayer(iPlayer).getCityList() :
            pCity = city.GetCy()
            RevUtils.changeCityRevolutionIndex( pCity, iRevIdxChange )


    def onCityAcquired( self, argsList):
	'City Acquired'
		
	owner,playerType,city,bConquest,bTrade = argsList

	owner = gc.getPlayer( city.getOwner() )
	prevOwner = gc.getPlayer( city.getPreviousOwner() )

        # Register loss with players other cities
	#self.playerCityLost( prevOwner, city, bConquest )

	if( owner.isBarbarian() ) :
            return
	
        newRevIdx = 400
        changeRevIdx = -40
        iPlayer = owner.getID()
        
        if( iPlayer == city.getOriginalOwner() ) :
            changeRevIdx -= 25
            newRevIdx -= 100

        if( city.getHighestPopulation() < 6 ) :
            changeRevIdx += 20
            newRevIdx -= 100

	if( bConquest ) :
            # Occupied cities also rack up rev points each turn
            newRevIdx += RevUtils.getCityRevolutionIndex( city )/4
            newRevIdx = min( [newRevIdx, 600] )

            if( city.plot().calculateCulturePercent( iPlayer ) > 90 ) :
                changeRevIdx -= 75
                newRevIdx -= 100
            elif( city.plot().calculateCulturePercent( iPlayer ) > 40 ) :
                changeRevIdx -= 35
                newRevIdx -= 60
            elif( city.plot().calculateCulturePercent( iPlayer ) > 20 ) :
                changeRevIdx -= 30
            
        elif( bTrade ) :
            newRevIdx += RevUtils.getCityRevolutionIndex( city )/4
            newRevIdx = min( [newRevIdx, 650] )

            if( city.plot().calculateCulturePercent( iPlayer ) > 90 ) :
                newRevIdx -= 50
            
        else :
            # Probably cultural conversion
            newRevIdx -= 100

            if( city.plot().calculateCulturePercent( iPlayer ) > 50 ) :
                changeRevIdx -= 25

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquisition of %s by %s reduces rev indices by %d"%(city.getName(),owner.getCivilizationDescription(0),changeRevIdx))

        changeRevIdx = int( self.cityAcquiredModifier*changeRevIdx )

        for listCity in PyPlayer( iPlayer ).getCityList() :
            pCity = listCity.GetCy()
            RevUtils.changeCityRevolutionIndex( pCity, changeRevIdx )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - New rev idx for %s is %d"%(city.getName(),newRevIdx))

        RevUtils.setCityRevolutionIndex( city, newRevIdx )
        RevUtils.setCityRevolutionCounter( city, self.acquiredTurns )

##################################################################################

    def updateRevolutionCounters( self, iPlayer ) :
            
        playerPy = PyPlayer( iPlayer )
        cityList = playerPy.getCityList()

        for city in cityList :
            pCity = city.GetCy()

            if( RevUtils.getCityRevolutionCounter( pCity ) > 0 ) :
                RevUtils.changeCityRevolutionCounter( pCity, -1 )

            if( RevUtils.getCityWarningCounter( pCity ) > 0 ) :
                RevUtils.changeCityWarningCounter( pCity, -1 )

    def updateLocalRevIndices( self, iPlayer ) :
        # Updates the revolution effects local to each city

        # Adjust index accumulation for varying game speeds
        gameSpeedMod = (661.0)/(game.getMaxTurns() + 1.0)
        if( gameSpeedMod == 661.0 ) : gameSpeedMod = 1.0
        #print "Revolt - Game speed mod is ", gameSpeedMod

        defNumPlayers = gc.getWorldInfo(gc.getMap().getWorldSize()).getDefaultPlayers()
	expCivPlots = CyMap().getLandPlots() / ( defNumPlayers * 1.0)

        pPlayer = gc.getPlayer(iPlayer)
        if( pPlayer.getNumCities() == 0 ) :
            return

        pTeam = gc.getTeam( pPlayer.getTeam() )
        playerPy = PyPlayer( iPlayer )
        cityList = playerPy.getCityList()

        hasNationalism = gc.getTeam(pPlayer.getTeam()).isHasTech(self.iNationalismTech)
        hasLiberalism = gc.getTeam(pPlayer.getTeam()).isHasTech(self.iLiberalismTech)
        hasSciMethod = gc.getTeam(pPlayer.getTeam()).isHasTech(self.iSciMethodTech)
        iCivNumLandPlots = pPlayer.getTotalLand()
        civSizeValue = self.civSizeModifier*iCivNumLandPlots / (1.0*expCivPlots)
        capital = pPlayer.getCapitalCity()

        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have civ size value %f"%(pPlayer.getCivilizationDescription(0),civSizeValue))

        for city in cityList :
            pCity = city.GetCy()
            
            prevRevIdx = RevUtils.getCityRevolutionIndex( pCity )

            # Sum up local factors for the city
            localRevIdx = 0

            # Happiness
            numUnhappy = RevUtils.getModNumUnhappy( pCity, self.warWearinessMod )
            if( numUnhappy > 0 ) :
                localRevIdx += int( 15*pow(numUnhappy, .7) )
            elif( numUnhappy <= 0 ) :
                numHappy = abs(numUnhappy)
                localRevIdx -= int( pow(numHappy, .6) + 1 )

            ww = pCity.getWarWearinessPercentAnger()

            if( ww > 0 and self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Rev - %s has ww %d"%(pCity.getName(),ww))

            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s happiness index: %d"%(pCity.getName(),localRevIdx))

            # Connected to capital
            if( not pCity.isConnectedTo(capital) ) :
                localRevIdx += 2

            # Distance to capital & Civ Size
            # Distance to captial divided by a little less than civ's "radius"
            cityDistModifier = plotDistance( pCity.getX(), pCity.getY(), capital.getX(), capital.getY() )/( pow(iCivNumLandPlots/3.2, .5) )
            cityDistModifier = cityDistModifier*(1+0.1*gc.getNumEraInfos()-0.1*pPlayer.getCurrentEra())
            cityDistModifier = min([cityDistModifier, 3.5])
            
            cityDistModifier = self.civSizeModifier*cityDistModifier

            locationRevIdx = 0

            if( civSizeValue > 2.0 ) :
                locationRevIdx += 4
                locationRevIdx += int(2.0*cityDistModifier)
            elif( civSizeValue > 1.6 ) :
                locationRevIdx += 2
                locationRevIdx += int(1.6*cityDistModifier)
            elif( civSizeValue > 1.4 ) :
                locationRevIdx += 2
                locationRevIdx += int(1.4*cityDistModifier)
            elif( civSizeValue > 1.2 ) :
                locationRevIdx += 1
                locationRevIdx += int(1.2*cityDistModifier)
            elif( civSizeValue > 1.0 ) :
                locationRevIdx += 0
                locationRevIdx += int(cityDistModifier)
            elif( civSizeValue > .7 ) :
                locationRevIdx += int(.7*cityDistModifier)
            else :
                locationRevIdx += int(.5*cityDistModifier)

            # Area
            if( not (pCity.area().getID() == capital.area().getID()) ) :
                if( hasNationalism ) :
                    locationRevIdx += int( self.colonyModifier*4 )
                else :
                    locationRevIdx += int( self.colonyModifier*2 )

            if( locationRevIdx > 0 ) :
                iGovernmentCivic = pPlayer.getCivics(self.iCivicOptionGovernment)
                if( iGovernmentCivic == self.iUniversalSuffrageID ) :
                    locationRevIdx = (2*locationRevIdx)/3
                elif( iGovernmentCivic == self.iRepresentationID ) :
                    locationRevIdx = (3*locationRevIdx)/4

            localRevIdx += int( locationRevIdx )

            # Civic choices

            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s location effects: %d  total: %d"%(pCity.getName(),locationRevIdx,localRevIdx))

            # Religion
            if( pPlayer.isStateReligion() ) :
                stateRel = pPlayer.getStateReligion()
                relGoodIdx = 0
                relBadIdx = 0
                for relType in range(0,gc.getNumReligionInfos()) :
                    if( pCity.isHasReligion(relType) ) :
                        if( relType == stateRel ) :
                            relGoodIdx += 3
                        else :
                            if( relBadIdx > 4 ) :
                                relBadIdx += 1
                            else :
                                relBadIdx += 2

                # Holy city
                if( pCity.isHolyCity() ) :
                    if( pCity.isHolyCityByType(stateRel) ) :
                        relGoodIdx += 5
                    else :
                        relBadIdx += 3

                iReligionCivic = pPlayer.getCivics(self.iCivicOptionReligion)
                if( iReligionCivic == self.iOrganizedReligionID ) :
                    relBadIdx = int(relBadIdx*2)
                elif( iReligionCivic == self.iTheocracyID ) :
                    relBadIdx = relBadIdx*4
                    relGoodIdx = int(relGoodIdx*1.5)

                if( pTeam.getAtWarCount(True) > 1 ) :
                    # God is on your side =P
                    relGoodIdx = int(relGoodIdx*1.5)

                relIdx = int( self.religionModifier*(relBadIdx - relGoodIdx) )

                if( hasSciMethod ) :
                    relIdx = relIdx/3
                elif( hasLiberalism ) :
                    relIdx = relIdx/2

                localRevIdx += relIdx
                if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s religious index: %d, total: %d"%(pCity.getName(),relIdx,localRevIdx))
                
            # Culture
            localRevIdx -= int( pow(abs(pCity.getCommerceRate(CommerceTypes.COMMERCE_CULTURE)/3), .6) )

            # Nationality
            natIdx = 0
            recentlyAcquired = (game.getGameTurn()-pCity.getGameTurnAcquired() < 30)
            culturePercent = 0
            for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( pPlayer.getTeam() == gc.getPlayer(idx).getTeam() ) :
                    culturePercent += pCity.plot().calculateCulturePercent(idx)
                    
            if( culturePercent > 90 ) :
                natIdx = -3
            elif( culturePercent > 70 ) :
                natIdx = -2
            elif( culturePercent < 50 ) :
                natIdx = 6
            elif( culturePercent < 40 ) :
                natIdx = 12
            elif( culturePercent < 20 ) :
                natIdx = 20

            if( recentlyAcquired ) :
                natIdx = natIdx/2

            if( not hasNationalism ) :
                natIdx = natIdx/2

            if( natIdx > 0 ) :
                iGovernmentCivic = pPlayer.getCivics(self.iCivicOptionGovernment)
                if( iGovernmentCivic == self.iUniversalSuffrageID ) :
                    natIdx = (2*natIdx)/3
                elif( iGovernmentCivic == self.iRepresentationID ) :
                    natIdx = (3*natIdx)/4

            localRevIdx += natIdx

            # Size
            if( pCity.getPopulation() < 6 ) :
                if( pCity.isCapital() and pPlayer.getCurrentEra() - game.getStartEra() > 2 ) :
                    # To help remove late game tiny civs
                    localRevIdx += 4
                else :
                    localRevIdx -= 7 - pCity.getPopulation()

            # Health
            numUnhealthy = (pCity.badHealth(false) - pCity.goodHealth())
            if( numUnhealthy > 0 ) :
                localRevIdx += int( 2*pow(numUnhealthy, .6) )

            # Garrison
            localRevIdx -= int( 2*pow(pCity.plot().getNumDefenders(iPlayer)/3, .5) - 1 )

            # Starving
            if( pCity.foodDifference(True) < 0 and abs(pCity.foodDifference(True)) > pCity.getFood() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City %s is starving!"%(pCity.getName()))
                localRevIdx += 100

            # Disorder
            if( pCity.isOccupation() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City %s is in Disorder!"%(pCity.getName()))
                if( game.getGameTurn() - pCity.getGameTurnAcquired() < 20 ) :
                    # Give recently acquired cities a break
                    localRevIdx += 5
                else :
                    localRevIdx += 20

            # Adjust for game speed
            localRevIdx = int( gameSpeedMod*localRevIdx + .5 )
            
            RevUtils.setCityLocalRevIndex( pCity, localRevIdx )
            RevUtils.changeCityRevolutionIndex( pCity, localRevIdx )

            if( RevUtils.getCityRevolutionIndex(pCity) < 0 ) :
                RevUtils.setCityRevolutionIndex( pCity, 0 )

            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - Local effects for %s (%s): %d with total %d"%(pCity.getName(),pPlayer.getCivilizationDescription(0),localRevIdx,RevUtils.getCityRevolutionIndex(pCity)))


    def updateCivRevIndices( self, iPlayer ) :
        # Update the revolution effects for the entire empire

        # Adjust index accumulation for varying game speeds
        gameSpeedMod = (661.0)/(game.getMaxTurns() + 1.0)
        if( gameSpeedMod == 661.0 ) : gameSpeedMod = 1.0
        #print "Revolt - Game speed mod is ", self.gameSpeedMod

        pPlayer = gc.getPlayer(iPlayer)
        if( pPlayer.getNumCities() == 0 ) :
            return

        pTeam = gc.getTeam( pPlayer.getTeam() )

        civRevIdx = 0

        # Finances
        finIdx = 0
        if( pTeam.getAtWarCount(True) == 0 ) :
            if( pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) > 50 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are at peace and have very high taxes, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)/2
                if( pPlayer.getGold() < 40 + 20*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 10 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in serious financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    finIdx += 2*(pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)
                elif( pPlayer.getGold() < 100 + 30*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 10 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)

        else :
            if( pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) > 70 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are at war and have very high taxes, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                finIdx += pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 70
                if( pPlayer.getGold() < 80 + 25*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 15 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in serious financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)
                elif( pPlayer.getGold() < 150 + 40*pPlayer.getCurrentEra() and pPlayer.calculateGoldRate() < 10 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in financial trouble, gold: %d, rate: %d, gold perc: %f"%(pPlayer.getCivilizationDescription(0), pPlayer.getGold(), pPlayer.calculateGoldRate(),pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD )))
                    finIdx += (pPlayer.getCommercePercent( CommerceTypes.COMMERCE_GOLD ) - 50)/2

        civRevIdx += finIdx

        # Civics
        civicIdx = 0
        iLaborCivic = pPlayer.getCivics(self.iCivicOptionLabor)
        if( iLaborCivic == self.iSlaveryID ) :
            if( pPlayer.canDoCivics(self.iEmancipationID) ) :
                civicIdx += 4
            else :
                civicIdx += 3
        elif( iLaborCivic == self.iSerfdomID ) :
            if( pPlayer.canDoCivics(self.iEmancipationID) ) :
                civicIdx += 2
            else :
                civicIdx += 1

        iGovernmentCivic = pPlayer.getCivics(self.iCivicOptionGovernment)
        if( iGovernmentCivic == self.iPoliceStateID ) :
            civicIdx -= 2
        elif( iGovernmentCivic == self.iUniversalSuffrageID ) :
            civicIdx -= 2
        elif( iGovernmentCivic == self.iDespotismID ) :
            if( pPlayer.canDoCivics(self.iRepresentationID) ) :
                civicIdx += 5
            else :
                civicIdx += 2
        elif( iGovernmentCivic == self.iHereditaryRuleID ) :
            if( pPlayer.canDoCivics(self.iRepresentationID) ) :
                civicIdx += 2
            else :
                civicIdx += 1

        civRevIdx += civicIdx

        # Helps catch tiny civs in late game
        if( pPlayer.getNumMilitaryUnits() > 2*pPlayer.getTotalPopulation() and pPlayer.getTotalPopulation()/pPlayer.getNumCities() < 4 ) :
            if( pPlayer.getCurrentEra() - game.getStartEra() > 2 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt -    The %s have way more troops than citizens"%(pPlayer.getCivilizationDescription(0)))
                civRevIdx += int( 4*pPlayer.getNumMilitaryUnits()/pPlayer.getTotalPopulation() )
        
        # Any civic or religion changes cause anarchy for non-spiritual civs
        # TODO: single turn anarchy doesn't show up at beginning or end of players turn
        if( pPlayer.isAnarchy() ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is in anarchy!"%(pPlayer.getCivilizationDescription(0)))
            civRevIdx += 50

        if( pPlayer.isGoldenAge() ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are in their golden age!"%(pPlayer.getCivilizationDescription(0)))
            civRevIdx -= 15

        civRevIdx = int( gameSpeedMod*civRevIdx + .5 )

        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s civ wide effects: %d, civics: %d, finances: %d"%(pPlayer.getCivilizationDescription(0),civRevIdx,civicIdx,finIdx))

        for city in PyPlayer(iPlayer).getCityList() :
            pCity = city.GetCy()
            RevUtils.changeCityRevolutionIndex( pCity, civRevIdx )


    def checkForRevolution( self, iPlayer ) :

        pPlayer = gc.getPlayer(iPlayer)
        if( pPlayer.getNumCities() == 0 ) :
            return

        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking %s for revolutions"%(pPlayer.getCivilizationDescription(0)))
        playerPy = PyPlayer( iPlayer )
        cityList = playerPy.getCityList()

        revReadyCities = list()
        revInstigatorCities = list()
        warnCities = list()

        capRevIdx = 0

        for city in cityList :
            pCity = city.GetCy()

            revIdx = RevUtils.getCityRevolutionIndex( pCity )
            prevRevIdx = RevUtils.getCityPrevRevIndex( pCity )
            localRevIdx = RevUtils.getCityLocalRevIndex( pCity )

            numUnhappy = RevUtils.getModNumUnhappy( pCity, self.warWearinessMod )
            if( numUnhappy > 0 ) :
                cityThreshold = max([int( self.revInstigatorThreshold - 2*self.revInstigatorThreshold*numUnhappy/pCity.getPopulation() ),self.revInstigatorThreshold/10])
            else :
                cityThreshold = self.revInstigatorThreshold

            if( pCity.isCapital() ) :
                capRevIdx = revIdx

            if( revIdx >= int( self.warnFrac*cityThreshold ) and RevUtils.getCityRevolutionCounter(pCity) == 0 ) :
                if(  RevUtils.getCityWarningCounter(pCity) == 0 ) :
                    # Warn human of impending revolution (note can't instigate on warning turn)
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  REVOLT - %s (%s) is over %d warning threshold in year %d!!!!"%(pCity.getName(),pPlayer.getCivilizationDescription(0),int( self.warnFrac*cityThreshold ),game.getGameTurnYear()))
                    warnCities.append(pCity)
                elif( revIdx > cityThreshold and prevRevIdx > cityThreshold ) :
                    # City meets instigator criteria
                    revInstigatorCities.append(pCity)
                
            if( revIdx > int(self.revReadyFrac*cityThreshold) and prevRevIdx > int(self.revReadyFrac*cityThreshold) and RevUtils.getCityRevolutionCounter(pCity) == 0 ) :
                # City meets revolution ready criteria
                revReadyCities.append( pCity )

            RevUtils.setCityPrevRevIndex( pCity, revIdx )

        instigator = None

        if( len(revInstigatorCities) > 0 ) :
            for pCity in revInstigatorCities :
                revIdx = RevUtils.getCityRevolutionIndex(pCity)
                odds = revIdx

                if( RevUtils.getCityLocalRevIndex( pCity ) < -10 ) :
                    odds = odds/4
                elif( RevUtils.getCityLocalRevIndex( pCity ) < 0 ) :
                    odds = odds/2

                if( RevUtils.getCityLocalRevIndex( pCity ) > self.badLocalThreshold ) :
                    odds = 2*odds

                if( RevUtils.getCityLocalRevIndex( pCity ) > max([int(2.5*self.badLocalThreshold),10]) ) :
                    odds = 3*odds

                if( RevUtils.getModNumUnhappy( pCity, self.warWearinessMod ) > 0 ) :
                    odds = max([ int(1.5*self.revInstigatorThreshold), odds, 2*revIdx ])

                odds = int( self.revolutionModifier*odds )

                # Do revolution?
                if( game.getSorenRandNum( self.iRevRoll, 'Revolt - do revolution?' ) < odds ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  REVOLT - %s (%s) has decided to launch a revolution with index %d and odds %f in year %d!!!!"%(pCity.getName(),pPlayer.getCivilizationDescription(0),revIdx,odds/(1.0*self.iRevRoll),game.getGameTurnYear()))
                    instigator = pCity
                    break

        if( not instigator == None ) :
            self.pickRevolutionStyle( pPlayer, instigator, revReadyCities )
        else :
            for pCity in warnCities :
                RevUtils.setCityWarningCounter( pCity, self.warnTurns )
                
            if( self.showHumanPopup(pPlayer.getID()) and len(warnCities) > 0 ) :
                pTeam = gc.getTeam( pPlayer.getTeam() )
                popup = PyPopup.PyPopup(bDynamic=False)
                popup.setHeaderString( 'Revolution in the air!' )
                bodStr = 'Grave news!  The citizens of '
                if( len(warnCities) == 1 ) :
                    bodStr += '%s '%(warnCities[0].getName())
                else :
                    for pCity in warnCities[:len(warnCities)-1] :
                        bodStr += '%s, '%(pCity.getName())
                    bodStr += 'and %s '%(warnCities[-1].getName())
                bodStr += ' are contemplating revolution.'
                bodStr += '  It may not yet be too late to maintain order.  Perhaps you should switch to less repressive civics or increase the happiness of the city.'
                bodStr += '  On the other hand, you could dispatch some more troops to the city to maintain control.'
                bodStr += '  If the city residents are unhappy, any improvement in their situation could quiet their dissent.'
                if( pTeam.getAtWarCount(True) > 0 ) :
                    bodStr += '\n\nWhile victory in our current wars may calm their cries, the shame of losing another city to our enemies could erode our support.'
                    bodStr += '  Carefully consider your chances for a quick victory.'
                else :
                    bodStr += '\n\nThe glory of conquering other lands might convince them to stay, if you have your eyes on foreign lands it might be time to strike.'
                    if( pPlayer.getCitiesLost() > 0 ) :
                        bodStr += "  Particularly, if you could regain some of the cities you might have lost earlier, the glory would be great."
                    bodStr += '  But be warned, this is only a temporary solution!'
                popup.setBodyString( bodStr )

                # Center camera on city
                CyCamera().JustLookAt( warnCities[0].plot().getPoint() )
                
                popup.launch()
                

    def topCivAdjustments( self ) :
        # Penalty on top score/power to help keep game even
        # Benefit for highest culture

        maxCultPlayer = -1
        maxCult = 0
        maxPowerPlayer = -1
        maxPower = 0

        for iPlayer in range(0,gc.getMAX_CIV_PLAYERS()) :
            pPlayer = gc.getPlayer( iPlayer )
            if( pPlayer.isAlive() and not pPlayer.getNumCities() == 0 ) :            
                if( pPlayer.getPower() > maxPower ) :
                    maxPower = pPlayer.getPower()
                    maxPowerPlayer = iPlayer
                if( pPlayer.countTotalCulture() > maxCult ) :
                    maxCult = pPlayer.countTotalCulture()
                    maxCultPlayer = iPlayer

        # Power - Increase revolution chance for top player
        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have most power"%(gc.getPlayer(maxPowerPlayer).getCivilizationDescription(0)))
        for city in PyPlayer(maxPowerPlayer).getCityList() :
            pCity = city.GetCy()
            RevUtils.changeCityRevolutionIndex( pCity, 3 )

        # Culture - Cut the dominant culture player some slack
        if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have most culture"%(gc.getPlayer(maxCultPlayer).getCivilizationDescription(0)))
        for city in PyPlayer(maxCultPlayer).getCityList() :
            pCity = city.GetCy()
            RevUtils.changeCityRevolutionIndex( pCity, -2 )

        # Score - Increase revolution chance for top player (don't pile on too much with top power)
        maxScorePlayer = game.getRankPlayer( 0 )
        if( maxScorePlayer == maxPowerPlayer ) :
            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s also have highest score"%(gc.getPlayer(maxScorePlayer).getCivilizationDescription(0)))
            for city in PyPlayer(maxScorePlayer).getCityList() :
                pCity = city.GetCy()
                RevUtils.changeCityRevolutionIndex( pCity, 1 )
        else :
            if( self.LOG_DEBUG and game.getGameTurn()%25 == 0 ) : CvUtil.pyPrint("  Revolt - %s have highest score"%(gc.getPlayer(maxScorePlayer).getCivilizationDescription(0)))
            for city in PyPlayer(maxScorePlayer).getCityList() :
                pCity = city.GetCy()
                RevUtils.changeCityRevolutionIndex( pCity, 3 )
                

    def checkForAssimilation( self ) :

        iWorldCities = game.getNumCivCities()
        iPlayersAlive = game.countCivPlayersAlive()

        iWorldPlots = gc.getMap().getLandPlots()

        avgNumCities = iWorldCities/iPlayersAlive
        avgNumPlots = iWorldPlots/iPlayersAlive

        for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
            playerI = gc.getPlayer( idx )
            if( playerI.isAlive() and playerI.getNumCities() > 0 and playerI.getNumCities() < 3 ) :
                if( playerI.getTotalLand() < avgNumPlots/3 ) :
                    #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s is very small"%(playerI.getCivilizationDescription(0)))
                    capital = playerI.getCapitalCity()

                    if( capital.area().getNumCities() < playerI.getNumCities() + 2 ) :
                        # On own island
                        continue

                    if( capital.plot().calculateCulturePercent(idx) > 90 ) :
                        # Well cultured city
                        continue

                    if( game.getGameTurn() > 100 ) :
                        if( game.getGameTurn() - capital.getGameTurnAcquired() > 15 ) :

                            iOdds = 0

                            iCultPlayer = capital.plot().calculateCulturalOwner()
                            if( not iCultPlayer == playerI.getID() ) :
                                iOdds += 3

                            if( capital.getPopulation() < 4 ) :
                                iOdds += 2

                            if( RevUtils.getCityRevolutionIndex(capital) > self.alwaysViolentThreshold ) :
                                iOdds += 10

                            if( game.getSorenRandNum( 100, 'Revolution: Assimilate' ) < iOdds ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilation!  %s considering assimilation"%(playerI.getCivilizationDescription(0)))                   
                                # Pick civ to assimilate too
                                assimPlayer = None
                                cultPlayer = gc.getPlayer( iCultPlayer )
                                #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has majority culture"%(cultPlayer.getCivilizationDescription(0)))
                                if( cultPlayer == None or cultPlayer.isNone() ) :
                                    # something very strange going on
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error in Assimilation! Cult player supposed to be player %d"%(iCultPlayer))
                                    continue

                                if( not iCultPlayer == idx ) :
                                    if( not cultPlayer.isHuman() ) :
                                        assimPlayer = cultPlayer
                                        
                                else :
                                    maxSecondCult = 0
                                    for j in range(0,gc.getMAX_CIV_PLAYERS()) :
                                        if( not j == idx ) :
                                            if( capital.plot().getCulture(j) > maxSecondCult ) :
                                                secCultPlayer = gc.getPlayer( j )
                                                maxSecondCult = capital.plot().getCulture(j)

                                    if( not secCultPlayer.isHuman() ) :
                                        if( maxSecondCult/(1.0*capital.plot().countTotalCulture()) > .2 ) :
                                            relations = playerI.AI_getAttitude( secCultPlayer.getID() )
                                            if( relations == AttitudeTypes.ATTITUDE_PLEASED or relations == AttitudeTypes.ATTITUDE_FRIENDLY ) :
                                                assimPlayer = secCultPlayer
                                            else :
                                                if( maxSecondCult/(1.0*capital.plot().countTotalCulture()) > .35 ) :
                                                    if( relations == AttitudeTypes.ATTITUDE_CAUTIOUS ) :
                                                        assimPlayer = secCultPlayer

                                if( not assimPlayer == None ) :
                                    # Assimlate!
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Assimilating with %s"%(assimPlayer.getCivilizationDescription(0)))

                                    if( playerI.canContact(game.getActivePlayer()) ) :
                                        mess = "The %s have decided to join the %s and give up their independence."%(playerI.getCivilizationDescription(0),cultPlayer.getCivilizationDescription(0))
                                        CyInterface().addImmediateMessage(mess,"")

                                    playerIPy = PyPlayer(idx)

                                    for city in playerIPy.getCityList() :
                                        assimPlayer.acquireCity( city.GetCy(), False, True )

                                    for unit in playerIPy.getUnitList() :
                                        if( unit.plot().getOwner() == assimPlayer.getID() ) :
                                            unit.gift()


    def checkReturnHumanLeader( self ) :

        if( not self.humanLeaderType == None ) :
            if( game.getAIAutoPlay() == 1 ) :
                humanPlayer = gc.getActivePlayer()
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Returning human chosen leader to control of %s"%(humanPlayer.getCivilizationDescription(0)))
                RevUtils.changeCiv(humanPlayer.getID(),humanPlayer.getCivilizationType(),self.humanLeaderType)
                self.humanLeaderType = None

                

###################################################################################

    def pickRevolutionStyle( self, pPlayer, instigator, revReadyCities ) :

        iPlayer = pPlayer.getID()
        
        revCities = list()
        revCities.append(instigator)
        revInCapital = instigator.isCapital()
        
        # Who will join them?  City must be either in area with instigator, or close to instigator but not in homeland
        # City must also be able to revolt now (not recently revolted)
        for pCity in revReadyCities :
            if( RevUtils.getCityRevolutionCounter(pCity) == 0 and not pCity.getID() == instigator.getID() ) :
                
                if( pCity.area().getID() == instigator.area().getID() ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is in the area, joining revolution"%(pCity.getName()))
                    revCities.append(pCity)
                    revInCapital = (revInCapital or pCity.isCapital())
                    
                elif( plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() ) <= self.closeRadius and not pCity.area().getID() == pPlayer.getCapitalCity().area().getID() ) :
                    # Catch cities on small island chains ... not in same area, but close and not in homeland
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is nearby, joining revolution"%(pCity.getName()))
                    revCities.append(pCity)
                    revInCapital = (revInCapital or pCity.isCapital())

        # Peaceful or violent?
        peaceful = True
        instRevIdx = RevUtils.getCityRevolutionIndex( instigator )
        instLocalIdx = RevUtils.getCityLocalRevIndex( instigator )

        if( instRevIdx > self.alwaysViolentThreshold ) :
            # Situation really bad
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, above always violent threshold")
            peaceful = False
        elif( instigator.getNumRevolts(iPlayer) == 0 ) :
            peaceful = True
        else :
            if( peaceful ) :
                modNumUnhappy = RevUtils.getModNumUnhappy( instigator, self.warWearinessMod )
                if( int(200*modNumUnhappy/instigator.getPopulation()) > game.getSorenRandNum( 100, 'Rev' ) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent due to Unhappiness")
                    peaceful = False
            if( peaceful and instLocalIdx > self.badLocalThreshold ) :
                # Situation deteriorating rapidly
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent due to rapidly deteriorating situation")
                peaceful = False
            if( peaceful and len(revCities) == 1 ) :
                peaceful = True
            elif( peaceful ) :
                lowerThresh = int( .8*self.alwaysViolentThreshold )
                if( pPlayer.getCivics(self.iCivicOptionGovernment) == self.iPoliceStateID ) :
                    lowerThresh -= int( .15*self.alwaysViolentThreshold )
                elif( pPlayer.getCivics(self.iCivicOptionLegal) == self.iFreeSpeechID ) :
                    lowerThresh += int( .10*self.alwaysViolentThreshold )
                elif( instLocalIdx > 0 ) :
                    lowerThresh -= int( .05*self.alwaysViolentThreshold )
                
                if( instRevIdx > lowerThresh ) :
                    odds = (100*(instRevIdx - lowerThresh))/(self.alwaysViolentThreshold-lowerThresh)
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds for violence are %d"%(odds))
                    if( odds > game.getSorenRandNum( 100, 'Rev' ) ) :
                        peaceful = False
            
                
        

        if( peaceful ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful revolution")

        # All of these have violent and peaceful paths
#-------- Check if instigator influence by other culture -> try to join
        if( self.culturalRevolution and instigator.plot().calculateCulturePercent(iPlayer) <= self.maxNationalityThreshold ) :
            #cultOwnerID = instigator.plot().calculateCulturalOwner()
            # calculateCulturalOwner rules out dead civs ...
            maxCulture = 30
            cultOwnerID = -1
            for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( instigator.plot().getCulture( idx ) > maxCulture ) :
                    maxCulture = instigator.plot().getCulture( idx )
                    cultOwnerID = idx
                
            if( cultOwnerID >= 0 and cultOwnerID < gc.getBARBARIAN_PLAYER() and not gc.getPlayer(cultOwnerID).getTeam() == pPlayer.getTeam() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has majority culture from other player %d, asking to join"%(instigator.getName(),cultOwnerID))
                cultCities = list()
                for pCity in revCities :
                    maxCulture = 30
                    cityCultOwnerID = -1
                    for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                        if( pCity.plot().getCulture( idx ) > maxCulture ) :
                            maxCulture = pCity.plot().getCulture( idx )
                            cityCultOwnerID = idx
                            
                    if( cityCultOwnerID == cultOwnerID ) :
                        cultCities.append(pCity)
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s also wants to go to other civ"%(pCity.getName()))

                cultPlayer = gc.getPlayer( cultOwnerID )

                bodStr = 'The city of %s'%(cultCities[0].getName())
                if( len(cultCities) > 1 ) :
                    bodStr += ', along with'
                    for pCity in cultCities[1:len(cultCities)-1] :
                        bodStr += ' %s,'%(pCity.getName())
                    bodStr += ' and %s, are'%(cultCities[-1].getName())
                else :
                    bodStr += ' is'

                if( peaceful ) :
                    # ask to join other civ, if denied get angrier
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, asking to join the %s"%(cultPlayer.getCivilizationDescription(0)))

                    pRevPlayer = None
                    isJoinWar  = False
                    joinPlayer = None

                    if( cultPlayer.isAlive() ) :
                        bodStr += ' clamoring to join the %s.'%(cultPlayer.getCivilizationDescription(0))
                        # Will only form pRevPlayer if human join player rebuffs revolutionaries
                        [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( cultCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = False )
                        joinPlayer = cultPlayer

                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Cult player is dead, trying to reform")
                        bodStr += ' longing to re-form the %s.'%(cultPlayer.getCivilizationDescription(0))
                        pRevPlayer = cultPlayer

                    bodStr += '  Demonstrations and riots have broken out in the streets!'
                    bodStr += '\n\nDenying their request will anger the people and perhaps fuel their revolutionary spirit.'
                    bodStr += "  While they won't take up arms now, if you deny their request you should take action to increase their happiness or bond with your culture and religion."
                    bodStr += "  If there is a next time, you may face an uprising!"
                    
                else :
                    # demand to join other civ, if denied, fight!
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, demanding to join the %s"%(cultPlayer.getCivilizationDescription(0)))

                    if( cultPlayer.isAlive() ) :
                        joinPlayer = cultPlayer
                        [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( cultCities, bJoinCultureWar = True, bReincarnate = True, bJoinRebels = True )
                        if( joinPlayer.getID() == pRevPlayer.getID() ) :
                            joinPlayer = None
                        else :
                            if( self.allowSmallBarbRevs and len(cultCities) == 1 ) :
                                if( instRevIdx < int(1.2*self.revInstigatorThreshold) ) :
                                    pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                                    isJoinWar = False
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Small, disorganized Revolution")
                    else :
                        pRevPlayer = cultPlayer
                        isJoinWar = False
                        joinPlayer = None
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Cult player is dead, trying to reform")

                    if( not joinPlayer == None ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, demanding to join the %s"%(joinPlayer.getCivilizationDescription(0)))
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - If denied, will form/join the %s, alive: %d"%(pRevPlayer.getCivilizationDescription(0),pRevPlayer.isAlive()))
                        bodStr += ' demanding to join the %s.'%(joinPlayer.getCivilizationDescription(0))
                        if( isJoinWar ) :
                            bodStr += '  If you do not let them, they will join the %s against you.'%(pRevPlayer.getCivilizationDescription(0))
                        else :
                            if( pRevPlayer.isBarbarian() ) :
                                bodStr += "  If you do not let them, their loosely organized rebel bands will rise up against you."
                            else :
                                bodStr += '  If you do not let them, they will rise up against you as the %s.'%(pRevPlayer.getCivilizationDescription(0))
                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Violent, demanding to join the %s"%(pRevPlayer.getCivilizationDescription(0)))
                        if( pRevPlayer.isBarbarian() ) :
                            bodStr += ' demanding independence even though they have no flag to rally around.'
                        elif( not pRevPlayer.isAlive() ) :
                            bodStr += ' demanding independence to re-form the %s.'%(pRevPlayer.getCivilizationDescription(0))
                        else :
                            bodStr += ' demanding to join the %s.'%(pRevPlayer.getCivilizationDescription(0))

                    bodStr += '  The citizens have been massing weapons and drilling in the countryside!'
                    bodStr += '\n\nIf you deny their request to leave, you will have to face the revolutionaries on the battlefield.'
                    if( not joinPlayer == None ) :
                        bodStr += '  The %s may interpret your actions as an act of war and respond accordingly!'%(joinPlayer.getCivilizationDescription(0))
                        bodStr += "  If they don't retaliate, your relations with %s will still suffer, and the cities will rise up "
                        if( pRevPlayer.isBarbarian() ) :
                            bodStr += "in loosely organized rebel bands."
                        else :
                            bodStr += "as the %s and fight you."%(pRevPlayer.getCivilizationDescription(0))
                    else :
                        if( pRevPlayer.isBarbarian() ) :
                            pass
                        elif( pRevPlayer.isAlive() ) :
                            bodStr += '  The cities will rise up and join your enemy, the %s.'%(pRevPlayer.getCivilizationDescription(0))

                    bodStr += '  Your troops will be surrounded by a hostile population and may take heavy losses or even defect.'
                    bodStr += '  The defenses of some of these cities may crumble immeadiately, other may be able to retain some semblence of control.'
                    bodStr += '  While war with your own cities will be ugly, it may be better than allowing cities you control to leave without a fight!'
                    bodStr += '  Carefully weigh your options.'

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(cultCities)))
                assert( len(cultCities) > 0 )
                self.makeRevolutionDecision( cultCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer, isJoinWar = isJoinWar, joinPlayer = joinPlayer )
                
                return

#-------- Check for religious revolution
        if( self.religiousRevolution and pPlayer.isStateReligion() ) :
            revRel = None
            stateRel = pPlayer.getStateReligion()
            
            # Check holy city instigating
            if( instigator.isHolyCity() ) :
            
                if( not instigator.isHolyCityByType(stateRel) ) :

                    for relType in range(0,gc.getNumReligionInfos()) :
                        if( instigator.isHolyCityByType(relType) and not stateRel == relType ) :
                            if( self.allowStateReligionToJoin or not instigator.isHasReligion(stateRel) ) :
                                revRel = relType
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Instigator is rival religion (%s) holy city"%(gc.getReligionInfo( revRel ).getDescription()))
                                break

            # Check for significant minority religion
            if( revRel == None  ) :
                if( self.allowStateReligionToJoin or not instigator.isHasReligion(stateRel) ) :
                    if( len(revCities) >= 3 ) :
                        # Must be large movement of like minded cities
                        maxCount = 0
                        maxCountRel = -1
                        for relType in range(0,gc.getNumReligionInfos()) :
                            if( instigator.isHasReligion(relType) and not stateRel == relType ) :
                                relCount = 0
                                for pCity in revCities :
                                    if( pCity.isHasReligion(relType) ) :
                                        if( pCity.isHasReligion(stateRel) ) :
                                            relCount += .5
                                        else :
                                            relCount += 1
                                if( relCount > maxCount ) :
                                    maxCount = relCount
                                    maxCountRel = relType

                        # Is the best good enough?
                        if( maxCount >= 3 ) :
                            revRel = maxCountRel
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Instigator and enough others have %s"%(gc.getReligionInfo( revRel ).getDescription()))

            # Decide how to revolt
            if( not revRel == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Religious revolution")

                relCities = list()
                for pCity in revCities :
                    if( pCity.isHasReligion(revRel) ) :
                        if( self.allowStateReligionToJoin or not pCity.isHasReligion(stateRel) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has %s"%(pCity.getName(),gc.getReligionInfo( revRel ).getDescription()))
                            relCities.append(pCity)

                if( peaceful ) :
                    # Check if there are civics they would like changed
                    iReligionCivic = pPlayer.getCivics(self.iCivicOptionReligion)
                    newRelCivic = None
                    if( iReligionCivic == self.iTheocracyID ) :
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change from Theocracy")
                        if( pPlayer.canDoCivics(self.iFreeReligionID) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Free Religion")
                            newRelCivic = self.iFreeReligionID
                        elif( pPlayer.canDoCivics(self.iPacifismID) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Pacifism")
                            newRelCivic = self.iPacifismID
                        elif( pPlayer.canDoCivics(self.iOrganizedReligionID) ) :
                             # Should always be able to at least switch to organized
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No better alternative than to ask for Organized Religion")
                            newRelCivic = self.iOrganizedReligionID
                        
                    elif( not iReligionCivic == self.iFreeReligionID and pPlayer.canDoCivics(self.iFreeReligionID) ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to Free Religion")
                        newRelCivic = self.iFreeReligionID

                    if( not newRelCivic == None ) :

                        bodStr = 'The city of %s'%(instigator.getName())
                        if( len(relCities) > 1 ) :
                            bodStr += ', along with'
                            for pCity in relCities[1:len(relCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' and %s, are'%(relCities[-1].getName())
                        else :
                            bodStr += ' is'

                        # Can't pay them enough so they don't feel oppressed
                            
                        if( iReligionCivic == self.iTheocracyID ) :
                            bodStr += ' requesting an end to your theocracy.'
                            bodStr += ' They have been oppressed for too long, and desire you to switch your religious civic policy to %s.'%(gc.getCivicInfo(newRelCivic).getDescription())
                        else :
                            bodStr += ' requesting that you adopt Freedom of Religion.'
                            bodStr += '  They feel your state sponsorship of %s impedes their ability to practice and spread the one true faith, %s.'%(gc.getReligionInfo(stateRel).getDescription(),gc.getReligionInfo(revRel).getDescription())
                        bodStr += '\n\nWhile they have not yet begun preparations for war, if you deny their request they may not put up with your current policies much longer.'
                        bodStr += '  Honoring their request is by far the quickest and easiest path to keeping them in your empire for the forseeable future.'

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(relCities)))
                        assert( len(relCities) > 0 )
                        
                        self.makeRevolutionDecision( relCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newRelCivic )
                        
                        return

                    # Ask for switch of state religions?
                    revRelCount = pPlayer.getHasReligionCount(revRel)
                    stateRelCount = pPlayer.getHasReligionCount(stateRel)
                    if( revRelCount > stateRelCount/4 ) :
                        
                        bodStr = 'The city of %s'%(instigator.getName())
                        if( len(relCities) > 1 ) :
                            bodStr += ', along with'
                            for pCity in relCities[1:len(relCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' and %s, are'%(relCities[-1].getName())
                        else :
                            bodStr += ' is'
                            
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change in state religion, %d practice new, %d practice state"%(revRelCount,stateRelCount))
                        totalRevIdx = 0
                        for pCity in relCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(17-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        #buyOffCost = (60 + 12*pPlayer.getCurrentEra())*len(relCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' requesting that you change your state religion to %s.'%(gc.getReligionInfo( revRel ).getDescription())
                        bodStr += '  Currently, %d of your cities have the new religion, while %d have your current state religion.'%(revRelCount,stateRelCount)
                        bodStr += '\n\nWhile they have not yet begun preparations for war, if you deny their request they may soon become more fervent in their desire for change.'
                        bodStr += '  Honoring their request is by far the quickest and easiest path to keeping them in your empire for the forseeable future, but may lead to some discontent in other cities.'
                        bodStr += '\n\nAlternatively, a large donation of %d to the religious leaders would cause them to settle down.'%(buyOffCost)

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(relCities),buyOffCost))
                        assert( len(relCities) > 0 )
                        self.makeRevolutionDecision( relCities, pPlayer, 'religion', peaceful, bodStr, newReligion = revRel, buyOffCost = buyOffCost )

                        return

                    # Ask for independence
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking for independence for religious reasons")

                    # Prune for only close cities?  Cities in area may be quite far away
                    indCities = list()
                    for pCity in relCities :
                        cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                        if( cityDist <= 2*self.closeRadius ) :
                            for iCity in indCities :
                                cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])

                            if( cityDist <= self.closeRadius ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in independence quest"%(pCity.getName()))
                                indCities.append(pCity)   

                    bodStr = 'The city of %s'%(instigator.getName())
                    if( len(indCities) > 1 ) :
                        bodStr += ', along with'
                        for pCity in indCities[1:len(indCities)-1] :
                            bodStr += ' %s,'%(pCity.getName())
                        bodStr += ' and %s, are'%(indCities[-1].getName())
                    else :
                        bodStr += ' is'

                    #buyOffCost = (60 + 12*pPlayer.getCurrentEra())*len(indCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                    totalRevIdx = 0
                    for pCity in indCities :
                        totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                    buyOffCost = totalRevIdx/(17-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                    if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                    buyOffCost = max( [buyOffCost,pPlayer.getGold()/12 + game.getSorenRandNum(50,'Rev')] )
                    bodStr += ' seeking independence so that they may freely worship thier faith, %s.'%(gc.getReligionInfo( revRel ).getDescription())
                    bodStr += '  They have lived in your %s oriented society for too long.'%(gc.getReligionInfo( pPlayer.getStateReligion() ).getDescription())
                    bodStr += '\n\nWhile they have not yet begun preparations for war, if you deny their request they may soon become more fervent in their desire for independence.'
                    bodStr += '  If you choose not to let them walk away, you should spread your religion to the cites and take other action to improve their situation.'
                    bodStr += '  Either that, or you may very well be faced with an uprising in the future!'
                    bodStr += '\n\nAlternatively, you could make a generous donation of %d to the religions leaders.  With this good faith gesture, they will drop their cause and quiet down.'%(buyOffCost)

                    [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = False, giveRelType = revRel )

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(indCities),buyOffCost))
                    assert( len(indCities) > 0 )
                    self.makeRevolutionDecision( indCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer = pRevPlayer, buyOffCost = buyOffCost)

                    return
                    
                else :
                    # Demand independence for religious reasons
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Demanding independence for religious reasons")

                    # Prune for only close cities?  Cities in area may be quite far away
                    indCities = list()
                    for pCity in relCities :
                        cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                        if( cityDist <= 2*self.closeRadius ) :
                            for iCity in indCities :
                                cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])

                            if( cityDist <= self.closeRadius ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in independence quest"%(pCity.getName()))
                                indCities.append(pCity)   

                    bodStr = 'The city of %s'%(instigator.getName())
                    if( len(indCities) > 1 ) :
                        bodStr += ', along with'
                        for pCity in indCities[1:len(indCities)-1] :
                            bodStr += ' %s,'%(pCity.getName())
                        bodStr += ' and %s, are'%(indCities[-1].getName())
                    else :
                        bodStr += ' is'

                    bodStr += ' demanding independence so that they may freely worship thier faith, %s.'%(gc.getReligionInfo( revRel ).getDescription())
                    bodStr += '  They have lived in your %s dominated society for too long!'%(gc.getReligionInfo( pPlayer.getStateReligion() ).getDescription())
                    bodStr += '\n\nIf you deny their request to leave, you will have to face the devout revolutionaries on the battlefield.'
                    bodStr += '  Your troops in these cities will be surrounded by a hostile population and may take heavy losses or even defect.'
                    bodStr += '  The defenses of some of these cities may crumble immeadiately, other may be able to retain some semblence of control.'
                    bodStr += '  While war with your own cities will be ugly, it may be better than allowing cities you control to leave without a fight!'
                    bodStr += '  Carefully weigh your options.'

                    [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = False, giveRelType = revRel )

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(indCities)))
                    assert( len(indCities) > 0 )
                    self.makeRevolutionDecision( indCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer = pRevPlayer)

                    return

# --------------- Special options for homeland revolutions
        if( instigator.area().getID() == pPlayer.getCapitalCity().area().getID() ) :
            # Revolution in homeland
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolution in homeland")

            if( self.civicRevolution ) :
                if( peaceful and (60 > game.getSorenRandNum(100, 'Revolt - emancipation request') or pPlayer.isCivic(self.iSlaveryID)) ) :
                     if( pPlayer.canDoCivics(self.iEmancipationID) and not pPlayer.isCivic(self.iEmancipationID) ):
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to emancipation")
                        newCivic = self.iEmancipationID

                        bodStr = 'The city of %s'%(instigator.getName())
                        if( len(revCities) > 1 ) :
                            bodStr += ', along with'
                            for pCity in revCities[1:len(revCities)-1] :
                                bodStr += ' %s,'%(pCity.getName())
                            bodStr += ' and %s, are'%(revCities[-1].getName())
                        else :
                            bodStr += ' is'

                        #buyOffCost = (50 + 10*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' requesting that you officially reject the practice of slavery and adopt Emancipation in your civic policy.'
                        bodStr += '  Mass demonstrations have broken out, bringing production in your cities to a halt!'
                        bodStr += '\n\nThe demonstrators have not yet turned to violence, and there is still time to bring the situation to a peaceful conclusion.'
                        bodStr += '  Honoring their request is by far the quickest and easiest path to keeping these cities in your empire for the forseeable future.'
                        bodStr += '  However, if you decide to keep your current practices, you may yet have time to improve other aspects of these cities so they will forget your condoning the slave trade.'
                        bodStr += '  You may also choose to bribe the agitators with %d gold, and they will drop their cause and quiet down.'%(buyOffCost)

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return

                if( pPlayer.isCivic(self.iSlaveryID) ) :
                    if( not peaceful and 50 > game.getSorenRandNum( 100, 'Revolt - do slave rebellion' ) ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Slave rebellion!!!")

                        slaveCities = list()
                        for pCity in revCities :
                            cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
                            if( cityDist <= 2*self.closeRadius ) :
                                for iCity in slaveCities :
                                    cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])

                                if( cityDist <= self.closeRadius ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in slave rebellion"%(pCity.getName()))
                                    slaveCities.append(pCity)  

                        bodStr = 'A slave rebellion has broken out in '
                        if( len(slaveCities) == 1 ) :
                            bodStr += '%s!'%(slaveCities[0].getName())
                        else :
                            for pCity in slaveCities[:len(slaveCities)-1] :
                                bodStr += '%s, '%(pCity.getName())
                            bodStr += 'and %s!'%(slaveCities[-1].getName())

                        if( pPlayer.canDoCivics(self.iEmancipationID) ) :
                            newCivic = self.iEmancipationID
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to emancipation")
                            bodStr += '  The slave are demanding you switch to Emancipation, or they will fight!'                            

                        elif( pPlayer.canDoCivics(self.iSerfdomID) ) :
                            newCivic = self.iSerfdomID
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to serfdom")
                            bodStr += '  The slave are demanding you switch to Serfdom, or they will fight!'

                        else :
                            newCivic = self.iTribalismID
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to tribalism")
                            bodStr += '  The slave are demanding you switch to Tribalism, or they will fight!'

                        bodStr += '\n\nIf you deny their request, disorganized slave bands rise up outside your cities!'
                        bodStr += '  But if you end their servitude, you will put many whip makers out of business.'

                        # Slaves always rise up as barbarians
                        pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                        assert(len(slaveCities) > 0)
                        self.makeRevolutionDecision( slaveCities, pPlayer, 'civics', peaceful, bodStr, pRevPlayer = pRevPlayer, newCivic = newCivic )

                        return

# -------------------- Special options for colonial revolutions
        else :
            # Revolution based in other area
            # These are special requests peaceful colonists may make
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolution in colony")

            foreignCities = list()
            capitalArea = pPlayer.getCapitalCity().area().getID()
            for pCity in revCities :
                if( not pCity.area().getID() == capitalArea ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is colony"%(pCity.getName()))
                    foreignCities.append( pCity )

            bodStr = 'The city of %s'%(instigator.getName())
            if( len(foreignCities) > 1 ) :
                bodStr += ', along with'
                for pCity in foreignCities[1:len(foreignCities)-1] :
                    bodStr += ' %s,'%(pCity.getName())
                bodStr += ' and %s, are'%(foreignCities[-1].getName())
            else :
                bodStr += ' is'
                        
            if( peaceful and self.civicRevolution ) :
                # Sufferage or representation
                if( not pPlayer.isCivic(self.iRepresentationID) and not pPlayer.isCivic(self.iUniversalSuffrageID) ) :
                    if( pPlayer.canDoCivics(self.iRepresentationID) ) :
                        
                        bodStr += ' requesting a say in your government.  As a colonies, they feel your government is not treating them fairly.'
                        if( pPlayer.canDoCivics(self.iUniversalSuffrageID) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to universal sufferage")
                            newCivic = self.iUniversalSuffrageID
                            bodStr += "  The colonists are protesting in the streets demanding Universal Sufferage!"
                        else :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to representation")
                            newCivic = self.iRepresentationID
                            bodStr += "  Cries of 'No taxation without Representation!' fill the air as colonists march!"

                        #buyOffCost = (75 + 12*pPlayer.getCurrentEra())*len(foreignCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in foreignCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(15-pPlayer.getCurrentEra()) + game.getSorenRandNum(80+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += '\n\nThe protesters have not yet turned to violence, and there is still time to bring the situation to a peaceful conclusion.'
                        bodStr += '  Honoring their request is by far the quickest and easiest path to keeping these cities in your empire for the forseeable future.'
                        bodStr += '  However, if you decide to keep your current practices, you may yet have time to improve other aspects of these cities so they will overlook their exclusion from power.'
                        bodStr += '  You can also choose to bribe the colonists with %d gold.  While not a permanent solution, they will drop their request and quiet down.'%(buyOffCost)

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( foreignCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return

            #if( peaceful and self.vassalRevolution ) :
                # Ask to become vassal
                # Trim down cities to only those close to instigator
            
                             
#-------- If capital or majority of cities,
        if( revInCapital or len(revCities) >= (pPlayer.getNumCities()+1)/3 ) :

            bodStr = 'The city of %s'%(instigator.getName())
            if( len(revCities) > 1 ) :
                bodStr += ', along with'
                for pCity in revCities[1:len(revCities)-1] :
                    bodStr += ' %s,'%(pCity.getName())
                bodStr += ' and %s, are'%(revCities[-1].getName())
            else :
                bodStr += ' is'
            
            if( peaceful ) :
                # If peaceful, ask change to civics ... if no civics, ask for change of leader
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Capital or large number of cities in peaceful revolution!")

                if( self.civicRevolution ) :
                    # Emancipation
                    if( pPlayer.canDoCivics(self.iEmancipationID) and not pPlayer.isCivic(self.iEmancipationID) ):
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to emancipation")
                        newCivic = self.iEmancipationID

                        #buyOffCost = (50 + 12*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += ' requesting that you officially reject the practice of slavery and adopt Emancipation in your civic policy.'
                        bodStr += '  Mass demonstrations have broken out, bringing production in your cities to a halt!'
                        bodStr += '\n\nThe demonstrators have not yet turned to violence, and there is still time to bring the situation to a peaceful conclusion.'
                        bodStr += '  Honoring their request is by far the quickest and easiest path to keeping these cities in your empire for the forseeable future.'
                        bodStr += '  However, if you decide to keep your current practices, you may yet have time to improve other aspects of these cities so they will forget your condoning the slave trade.'
                        bodStr += "  You can also choose to bribe the agitators with a payment of %d.  While they won't use it to buy slaves, they will drop their cause and quiet down."%(buyOffCost)

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                        assert( len(revCities) > 0 )
                        self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                        return

                    if( not pPlayer.isCivic(self.iStatePropertyID) ) :
                        if( pPlayer.canDoCivics(self.iStatePropertyID) and 35 > game.getSorenRandNum(100, 'Rev - Communist revolution') ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Communist revolution, ask change to State Property")
                            newCivic = self.iStatePropertyID
                            
                            bodStr += ' requesting a switch to State Property.  They are imagining no possesions and a brotherhood of man!'
                            bodStr += '\n\nWhile State Property may have its benefits, it may not be worth giving up your current economic model.'
                            bodStr += '  The communist agitators have not yet turned to violence, so you may have time to help them see the benefits of competition.'
                            bodStr += '  However, the quickest and easiest path to keeping these cities in your empire for the forseeable future is to declare yourself a Comrade.'
                            bodStr += "  Since they don't value your cash, you cannot pay them off."

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution"%(len(revCities)))
                            assert( len(revCities) > 0 )
                            self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic )

                            return

                    # Sufferage or representation
                    if( not pPlayer.isCivic(self.iRepresentationID) and not pPlayer.isCivic(self.iUniversalSuffrageID) ) :
                        if( pPlayer.canDoCivics(self.iRepresentationID) ) :
                            
                            bodStr += ' requesting a say in your government.'
                            if( pPlayer.canDoCivics(self.iUniversalSuffrageID) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to universal sufferage")
                                newCivic = self.iUniversalSuffrageID
                                bodStr += "  Both men and women are protesting in the streets demanding Universal Sufferage!"
                            else :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking change to representation")
                                newCivic = self.iRepresentationID
                                bodStr += "  Cries of 'No taxation without Representation!' fill the air as protesters march!"

                            #buyOffCost = (50 + 15*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(150+15*pPlayer.getCurrentEra(),'Rev')
                            totalRevIdx = 0
                            for pCity in revCities :
                                totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                            buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                            buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
                            bodStr += '\n\nThe protesters have not yet turned to violence, and there is still time to bring the situation to a peaceful conclusion.'
                            bodStr += '  Honoring their request is by far the quickest and easiest path to keeping these cities in your empire for the forseeable future.'
                            bodStr += '  However, if you decide to keep your current practices, you may yet have time to improve other aspects of these cities so they will overlook their exclusion from power.'
                            bodStr += "  You can also choose to give them a generous bribe of %d, and they will drop their cause and quiet down.  Some new, expensive statues may remind them how great you are."%(buyOffCost)

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                            assert( len(revCities) > 0 )
                            self.makeRevolutionDecision( revCities, pPlayer, 'civics', peaceful, bodStr, newCivic = newCivic, buyOffCost = buyOffCost )

                            return

                if( self.leaderRevolution ) :
                    # Ask for change of leader
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Asking for change in leader from %s"%(pPlayer.getName()))

                    [newLeaderType,newLeaderName] = self.chooseRevolutionLeader( revCities )

                    bodStr += ' calling for a change in leadership!'
                    if( not pPlayer.isCivic(self.iRepresentationID) and not pPlayer.isCivic(self.iUniversalSuffrageID) ) :
                        isElection = False
                        #buyOffCost = (50 + 15*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(20-pPlayer.getCurrentEra()) + game.getSorenRandNum(80+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/10 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += '  Demonstrations have broken out, asking that you cede power to %s.'%(newLeaderName)
                        bodStr += '\n\nThe protesters have not yet turned to violence, and there is still time to bring the situation to a peaceful conclusion.'
                        bodStr += '  Like all good despots, you are of course the best hope for your people.  You just need to remind them of the glory of your rule, either by improving their situation, vanquishing your enemies, or moving lots of soldiers into their towns.'
                        bodStr += '  Spending %d on shiny statues of yourself will also quiet the dissenters.'%(buyOffCost)
                        bodStr += '\n\nHowever, if you cling to power now, you may face a civil war in the future!'
                        bodStr += '  Honoring their request would bring only a temporary end to your rule, as there is every reason to believe your heirs will regain power in the future.'
                        bodStr += '  %s could not possibly be half the ruler you are.  (If you take this option, your civ will be automated for some number of turns, then peacefully returned to your control)'%(newLeaderName)

                    else :
                        isElection = True
                        #buyOffCost = (30 + 10*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(25-pPlayer.getCurrentEra()) + game.getSorenRandNum(90+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/12 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += '  Demonstrations have broken out, asking that you face %s in an election!'%(newLeaderName)
                        bodStr += '\n\nThe protesters have not yet turned to violence, and there is still time to bring the situation to a peaceful conclusion.'
                        bodStr += '  Honoring their request is by far the quickest and easiest path to keeping these cities in your empire for the forseeable future.'
                        bodStr += '\n\nIf you win a fair and open election, the protests will disappear and your power will not be questioned.'
                        bodStr += "  It may also be possible to guarantee you 'victory', if you catch my drift, but it will be costly (%d gold) and the slight could be detected with very negative consequences."%(buyOffCost)
                        bodStr += '  Denying their request for an election will call into question your committment to open government and make your opponents more likely to fight in the future!'

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                    assert( len(revCities) > 0 )
                    self.makeRevolutionDecision( revCities, pPlayer, 'leader', peaceful, bodStr, newLeaderType = newLeaderType, newLeaderName = newLeaderName, isElection = isElection, buyOffCost = buyOffCost )

                    return
                
            else :
                # If violent, demand change of leader!
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Capital or majority of cities in violent revolution!")

                if( self.leaderRevolution ) :

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Demanding change in leader from %s"%(pPlayer.getName()))

                    [newLeaderType,newLeaderName] = self.chooseRevolutionLeader( revCities )

                    [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( revCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = True )

                    bodStr += ' demanding a change in leadership!'
                    if( not pPlayer.isCivic(self.iRepresentationID) and not pPlayer.isCivic(self.iUniversalSuffrageID) ) :
                        isElection = False
                        buyOffCost = -1
                        bodStr += '  Armies are gathering in the countryside, readying for a fight if you do not cede power to %s.'%(newLeaderName)
                        bodStr += '\n\nThis is truly a terrible situation.'
                        bodStr += '  Like all good despots, you are of course the best hope for your people, but some how they simply cannot see the light.'
                        bodStr += '  Now you must decide what legacy you will leave.'
                        bodStr += '\n\nYou could meet the challenge with force, and your cities would rise up as the %s to fight.'%(pRevPlayer.getCivilizationDescription(0))
                        bodStr += '  Your troops in these cities will be surrounded by a hostile population and may take heavy losses or even defect.'
                        bodStr += '  The defenses of some of these cities may crumble immeadiately, other may be able to retain some semblence of control.'
                        bodStr += '  If you then managed to reunite your country, the glory would be tremendous.'
                        bodStr += '  But if you fail, you will long be villified as your obstinance will have brought civil war into your homeland!'
                        bodStr += '\n\nThere is no glory in surrendering to the will of the people, but it would preserve all that you have built.'
                        bodStr += '  Honoring their request would bring only a temporary end to your rule, as there is every reason to believe your heirs will regain power in the future.'
                        bodStr += '  %s could not possibly be half the ruler you are.  (If you take this option, your civ will be automated for some number of turns, then peacefully returned to your control)'%(newLeaderName)

                    else :
                        isElection = True
                        #buyOffCost = (50 + 12*pPlayer.getCurrentEra())*len(revCities) + game.getSorenRandNum(100+10*pPlayer.getCurrentEra(),'Rev')
                        totalRevIdx = 0
                        for pCity in revCities :
                            totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
                        buyOffCost = totalRevIdx/(22-pPlayer.getCurrentEra()) + game.getSorenRandNum(80+10*pPlayer.getCurrentEra(),'Rev')
                        if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
                        buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
                        bodStr += '  Demonstrators are rioting in the streets, demanding that you face %s in an election!'%(newLeaderName)
                        bodStr += '\n\nIf you deny their request, these cities will rise up as the %s and fight.'%(pRevPlayer.getCivilizationDescription(0))
                        bodStr += '  Your troops in these cities will be surrounded by a hostile population and will take heavy losses or even defect.'
                        bodStr += '  The defenses of some of these cities may crumble immeadiately, other may be able to retain some semblence of control.'
                        bodStr += '\n\nHowever, if you win a fair and open election, the riots will disappear and your power will not be questioned.'
                        bodStr += "  It may also be possible to guarantee you 'victory', if you catch my drift, but it will be costly (%d gold) and the slight could be detected with very negative consequences."%(buyOffCost)

                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(revCities),buyOffCost))
                    assert( len(revCities) > 0 )
                    self.makeRevolutionDecision( revCities, pPlayer, 'leader', peaceful, bodStr, pRevPlayer = pRevPlayer, isJoinWar = isJoinWar, newLeaderType = newLeaderType, newLeaderName = newLeaderName, isElection = isElection, buyOffCost = buyOffCost )
                    
                    return

#-------- Default to ask/demand independence
        if( not self.independenceRevolution ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - WARNING: default ask for independence has been disabled!")
            return
        
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Default: ask/demand independence!")

        # Prune for only close cities?  Cities in area may be quite far away
        indCities = list()
        for pCity in revCities :
            cityDist = plotDistance( pCity.getX(), pCity.getY(), instigator.getX(), instigator.getY() )
            # Must be within 2 dists of instigator
            if( cityDist <= 2*self.closeRadius ) :
                for iCity in indCities :
                    cityDist = min([cityDist, plotDistance( pCity.getX(), pCity.getY(), iCity.getX(), iCity.getY() )])
                # Must be close to another revolutionary
                if( cityDist <= self.closeRadius ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is close enough to join in independence quest"%(pCity.getName()))
                    indCities.append(pCity)                

        bodStr = 'The city of %s'%(instigator.getName())
        if( len(indCities) > 1 ) :
            bodStr += ', along with'
            for pCity in indCities[1:len(indCities)-1] :
                bodStr += ' %s,'%(pCity.getName())
            bodStr += ' and %s, are'%(indCities[-1].getName())
        else :
            bodStr += ' is'

        buyOffCost = -1

        if( peaceful ) :
            [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = True )
            #buyOffCost = (100 + 20*pPlayer.getCurrentEra())*len(indCities) + game.getSorenRandNum(100+20*pPlayer.getCurrentEra(),'Rev')
            totalRevIdx = 0
            for pCity in indCities :
                totalRevIdx += RevUtils.getCityRevolutionIndex( pCity )
            buyOffCost = totalRevIdx/(17-pPlayer.getCurrentEra()) + game.getSorenRandNum(50+10*pPlayer.getCurrentEra(),'Rev')
            if( not pPlayer.isHuman() ) : buyOffCost = int( buyOffCost*.7 )
            buyOffCost = max( [buyOffCost,pPlayer.getGold()/8 + game.getSorenRandNum(50,'Rev')] )
            bodStr += ' asking for independence.  These citizens feel your empire is not meeting their needs and wish to strike out on their own and form the %s.'%(pRevPlayer.getCivilizationDescription(0))
            bodStr += '\n\nWhile they are not yet contemplating war, if you deny their request you should take immeadiate action to improve their situation.'
            bodStr += '  Increasing culture or strengthening the garrisons may lessen their resolve.'
            bodStr += '\n\nYou can also choose to payoff the revolutionaries.  In exchange for %d, the revolutionaries will drop their grievances for the time being.'%(buyOffCost)
            bodStr += '  Be warned, however, other cities may be more inclined to rebel after seeing you hand out money.'
            
        else :
            [pRevPlayer,isJoinWar] = self.chooseRevolutionCiv( indCities, bJoinCultureWar = False, bReincarnate = True, bJoinRebels = True )
            if( self.allowSmallBarbRevs and len(indCities) == 1 ) :
                if( instRevIdx < int(1.2*self.revInstigatorThreshold) ) :
                    pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                    isJoinWar = False
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Small, disorganized Revolution")
            bodStr += ' demanding independence.  They have had enough of you and your empire, and will fight '
            if( pRevPlayer.isBarbarian() ) :
                bodStr += "even though they have no flag to rally around."
            else :
                bodStr += 'if necessary to form the %s.'%(pRevPlayer.getCivilizationDescription(0))
            bodStr += '\n\nThis is a difficult situation, as your options are to give up some of your territory, or face civil war!'
            bodStr += '  Neither option is very appealing, you must weigh your military preparedness against the value of these cities to your empire.'
            bodStr += '  If you choose war, your troops in these cities will be surrounded by a hostile population and will take heavy losses.'
            bodStr += '  The defenses of some of these cities may crumble immeadiately, others may be able to retain some semblence of control.'

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %d cities in revolution, buyoff cost %d"%(len(indCities),buyOffCost))
        assert( len(indCities) > 0 )
        self.makeRevolutionDecision( indCities, pPlayer, 'independence', peaceful, bodStr, pRevPlayer = pRevPlayer, isJoinWar = isJoinWar, buyOffCost = buyOffCost )

        return


######################################################################

    def chooseRevolutionCiv( self, cityList, bJoinCultureWar = True, bReincarnate = True, bJoinRebels = True, pNotThisCiv = None, giveTechs = True, giveRelType = -1 ) :
        
        pRevPlayer = None
        isJoinWar = False

        # Attempt to find a worthy civ to reincarnate in any of these cities
        for pCity in cityList :

            if( pRevPlayer == None ) :

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Looking for revolution worthy civ in %s"%(pCity.getName()))

                owner = gc.getPlayer( pCity.getOwner() )
                ownerTeam = gc.getTeam( owner.getTeam() )

                cultPlayer = None
                if( pCity.countTotalCultureTimes100() > 50*100 ) :
                    cultPlayer = gc.getPlayer( pCity.findHighestCulture() )
                    if( cultPlayer.getID() == owner.getID() ) :
                        cultPlayer = None

                if( bJoinCultureWar and not cultPlayer == None ) :
                    # If at war with significant culture, join them
                    if( ownerTeam.getAtWarCount(True) > 0 ) :
                        if( ownerTeam.isAtWar(cultPlayer.getID()) and cultPlayer.isAlive() ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner at war with dominant culture player %s"%(cultPlayer.getCivilizationDescription(0)))
                            pRevPlayer = cultPlayer
                            if( cultPlayer.getNumCities() > 1 ) :
                                isJoinWar = True

                if( bJoinRebels and pRevPlayer == None ) :
                    # If at war with this cities rebel civ type, join them
                    revCivType = RevUtils.getCityRevolutionCiv( pCity )
                    if( revCivType >= 0 ) :
                        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                            if( not i == owner.getID() ) :
                                playerI = gc.getPlayer( i )
                                if( playerI.isAlive() and playerI.getCivilizationType() == revCivType ) :
                                    if( ownerTeam.isAtWar(playerI.getTeam()) ) :
                                        pRevPlayer = playerI
                                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner at war with cities revolt civ type, player %s"%(pRevPlayer.getCivilizationDescription(0)))

                    if( pRevPlayer == None ) :
                        # Look for other rebellions we could join?
                        pass

                if( bReincarnate and pRevPlayer == None ) :
                    # Check for civ that can rise from the ashes
                    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                        if( not i == owner.getID() ) :
                            playerI = gc.getPlayer( i )
                            if( (not playerI.isAlive()) and (pCity.getCulture( i ) > 50) ) :
                                pRevPlayer = playerI
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reincarnating player %s"%(pRevPlayer.getCivilizationDescription(0)))
                                break
                            
        # Search around all cities in list for a dead player to reincarnate
        if( pRevPlayer == None and bReincarnate ) :
            if( game.countCivPlayersAlive() < game.countCivPlayersEverAlive() ) :

                deadCivs = list()
                for idx in range(0,gc.getMAX_CIV_PLAYERS()) :
                    if( not gc.getPlayer(idx).isAlive() ) :
                        deadCivs.append(idx)
                        
                for pCity in cityList :
                    if(  pRevPlayer == None ) :
                        
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Checking around %s for dead player"%(pCity.getName()))

                        # Search plots (in case city was destroyed) around this city
                        maxCult = 0
                        radius = 1
                        gameMap = gc.getMap()
                        while( radius < 5 ) :
                            for ix in range(pCity.getX()-radius,pCity.getY()+radius+1) :
                                for iy in [pCity.getY() - radius, pCity.getY() + radius] :

                                    if( ix < 0 ) :
                                        if( gameMap.isWrapX() ) :
                                            ix = CyMap().getGridWidth() + ix
                                        else :
                                            continue
                                    elif( ix >= CyMap().getGridWidth() ) :
                                        if( gameMap.isWrapX() ) :
                                            ix = ix - CyMap().getGridWidth()
                                        else :
                                            continue

                                    if( iy < 0 ) :
                                        if( gameMap.isWrapY() ) :
                                            iy = CyMap().getGridHeight() + iy
                                        else :
                                            continue
                                    elif( iy >= CyMap().getGridHeight() ) :
                                        if( gameMap.isWrapY() ) :
                                            iy = iy - CyMap().getGridHeight()
                                        else :
                                            continue

                                    plotI = gc.getMap().plot(ix,iy)

                                    for civIdx in deadCivs :
                                        if( plotI.getCulture(civIdx) > maxCult ) :
                                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("Revolt: Found plot culture from dead player %d"%(civIdx))
                                            maxCult = plotI.getCulture(civIdx)
                                            pRevPlayer = gc.getPlayer(civIdx)

                            if( pRevPlayer == None ) :
                                radius += 1
                            else :
                                break

        # Create new civ based on culture/owner of first city in list
        if( pRevPlayer == None ) :

            pCity = cityList[0]
            owner = gc.getPlayer( pCity.getOwner() )
            ownerTeam = gc.getTeam( owner.getTeam() )
            
            # Search for empty slot
            newPlayerIdx = -1
            # Overwrite a civ that never managed to found a city and hasn't lost a city (probably a previous rebel)
            for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                if( (not gc.getPlayer(i).isAlive()) and (not gc.getPlayer(i).isFoundedFirstCity()) and gc.getPlayer(i).getCitiesLost() == 0 and (gc.getPlayer(i).getNumUnits() == 0) ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Dead player %d has founded city? %d Num cities lost: %d"%(i,gc.getPlayer(i).isFoundedFirstCity(),gc.getPlayer(i).getCitiesLost()))
                    newPlayerIdx = i
                    break

            if( newPlayerIdx < 0 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No available slots, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]

            # Create list of available civs and similar civ types
            
            # Don't incarnate as either of these
            iMinor = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_MINOR')
            iBarbarian = CvUtil.findInfoTypeNum(gc.getCivilizationInfo,gc.getNumCivilizationInfos(),'CIVILIZATION_BARBARIAN')
            # Civs not currently in the game
            availableCivs = list()
            # Civs with similar style to cultOwner, if they exist
            similarStyleCivs = list()
            similarOwnerStyleCivs = list()
            for civType in range(0,gc.getNumCivilizationInfos()) :
                if( not civType == iBarbarian ) :
                    if( not civType == iMinor ) :
                        taken = False
                        for i in range(0,gc.getMAX_CIV_PLAYERS()) :
                            if( civType == gc.getPlayer(i).getCivilizationType() ) :
                                if( gc.getPlayer(i).isAlive() or gc.getPlayer(i).isFoundedFirstCity() or gc.getPlayer(i).getCitiesLost() > 0 ) :
                                    taken = True
                                    break
                        if( not taken ) :
                            availableCivs.append(civType)
                            if( not cultPlayer == None ) :
                                if( gc.getCivilizationInfo( cultPlayer.getCivilizationType() ).getArtStyleType() == gc.getCivilizationInfo(civType).getArtStyleType() ) :
                                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential similar style civ from culture: %s (%d)"%(gc.getCivilizationInfo(civType).getShortDescription(0),civType))
                                    similarStyleCivs.append(civType)
                            if( gc.getCivilizationInfo( owner.getCivilizationType() ).getArtStyleType() == gc.getCivilizationInfo(civType).getArtStyleType() ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential similar style civ from owner: %s (%d)"%(gc.getCivilizationInfo(civType).getShortDescription(0),civType))
                                similarOwnerStyleCivs.append(civType)

            if( len(availableCivs) < 1 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No available civs, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]
            
            # If city has a revolutionary civ type, if that type is available choose it
            if( RevUtils.getCityRevolutionCiv(pCity) in availableCivs ) :
                # City previously rebelled as available civ type
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Respawning previous rebel civ type for this city")
                newCivIdx = RevUtils.getCityRevolutionCiv( pCity )
            else :
                if( len(similarStyleCivs) > 0 ) :
                    # Create a civ using style of cultural owner of city
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Creating a similar style civ to cultural owner %s"%(cultPlayer.getCivilizationDescription(0)))
                    availableCivs = similarStyleCivs
                elif( len(similarOwnerStyleCivs) > 0 ) :
                    # Create a civ using style of owner of city
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Creating a similar style civ to owner %s"%(owner.getCivilizationDescription(0)))
                    availableCivs = similarOwnerStyleCivs
                    
                newCivIdx = availableCivs[game.getSorenRandNum(len(availableCivs),'Revolution: pick unused civ type')]

            leaderList = list()
            for leaderType in range(0,gc.getNumLeaderHeadInfos()) :
                if( gc.getCivilizationInfo(newCivIdx).isLeaders(leaderType) ) :
                    leaderList.append(leaderType)

            if( len(leaderList) < 1 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unexpected lack of possible leaders, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]
            
            newLeaderIdx = leaderList[game.getSorenRandNum(len(leaderList),'Revolution: pick leader')]

            success = game.createNewPlayer( newPlayerIdx, newCivIdx, newLeaderIdx, -1, False, 1, 1 )

            if( not success ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - New civ creation failed, spawning as Barbarians")
                pRevPlayer = gc.getPlayer( gc.getBARBARIAN_PLAYER() )
                return [pRevPlayer, isJoinWar]

            pRevPlayer = gc.getPlayer(newPlayerIdx)
            
        # Do special setup for non-living revolutionaries ...
        
        if( giveTechs and not pRevPlayer.isAlive() ) :
            RevUtils.giveTechs( pRevPlayer, owner )

        if( not giveRelType == None and not pRevPlayer.isAlive() ) :
            if( giveRelType >= 0 ) :
                # Give specified religion
                pRevPlayer.setLastStateReligion( giveRelType )
                if( self.LOG_DEBUG ) :
                    religionDesc = gc.getReligionInfo( pRevPlayer.getStateReligion() ).getDescription()
                    CvUtil.pyPrint("  Revolt - Revolutionary's religion set to %s"%(religionDesc))
            else :
                # Give minority religion in city
                availRels = list()
                for relType in range(0,gc.getNumReligionInfos()) :
                    if( not relType == owner.getStateReligion() ) :
                        if( pCity.isHolyCityByType(relType) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s is holy city for non-state religion %s"%(pCity.getName(), gc.getReligionInfo( relType ).getDescription()))
                            giveRelType = relType
                        elif( pCity.isHasReligion(relType) ) :
                            availRels.append( relType )

                if( giveRelType < 0 ) :
                    if( len(availRels) > 0 ) :
                        giveRelType = availRels[game.getSorenRandNum(len(availRels),'Revolution: pick religion')]

                if( giveRelType >= 0 ) :
                    pRevPlayer.setLastStateReligion( giveRelType )
                    if( self.LOG_DEBUG ) :
                        religionDesc = gc.getReligionInfo( pRevPlayer.getStateReligion() ).getDescription()
                        CvUtil.pyPrint("  Revolt - Revolutionary's religion set to %s"%(religionDesc))

        return [pRevPlayer, isJoinWar]


    def chooseRevolutionLeader( self, cityList ) :

        newLeaderType = None
        newLeaderName = None

        owner = gc.getPlayer( cityList[0].getOwner() )
        ownerCivType = owner.getCivilizationType()
        ownerLeaderType = owner.getLeaderType()
        ownerCivInfo = gc.getCivilizationInfo( ownerCivType )

        # Use new leader type
        count = 0
        availLeader = list()
        for i in range(0,gc.getNumLeaderHeadInfos()) :
            if( ownerCivInfo.isLeaders(i) ) :
                count += 1
                if( not i == ownerLeaderType ) :
                    availLeader.append(i)

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Found %d available from %d for civ"%(len(availLeader),count))

        if( len(availLeader) > 0 ) :
            newLeaderType = availLeader[game.getSorenRandNum(len(availLeader),'Revolution: pick leader')]
            newLeaderName = gc.getLeaderHeadInfo( newLeaderType ).getDescription()
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Switching from leader type %d to %d (%s)"%(ownerLeaderType,newLeaderType,newLeaderName))

        if( newLeaderType == None ) :
            # Use same leader type, but with new name
            # TODO: Allow change of leader name
            # Hack for right now
            newLeaderType = ownerLeaderType
            newLeaderName = gc.getLeaderHeadInfo( newLeaderType ).getDescription()
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No new leader available, using current type %d (%s)"%(newLeaderType,newLeaderName))

        return [newLeaderType,newLeaderName]            

    def makeRevolutionDecision( self, cityList, pPlayer, revType, peaceful, bodStr, pRevPlayer = None, isJoinWar = False, joinPlayer = None, newLeaderType = None, newLeaderName = None, isElection = True, newCivic = None, newReligion = None, buyOffCost = -1 ) :

        pTeam = gc.getTeam( pPlayer.getTeam() )

        iAggressive = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
        iSpiritual = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_SPIRITUAL')
        numRevCities = len(cityList)

        if( revType == 'civics' ) :
            if( newCivic == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no civic specified")
                return

            if( peaceful ) :
                if( numRevCities > pPlayer.getNumCities()/2 ) :
                    odds = 90
                elif( numRevCities < pPlayer.getNumCities()/4 ) :
                    odds = 30
                else :
                    odds = 50
            else :
                if( numRevCities > pPlayer.getNumCities()/2 ) :
                    odds = 100
                elif( numRevCities < pPlayer.getNumCities()/4 ) :
                    odds = 50
                else :
                    odds = 70

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on civics")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost

                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( peaceful ) :
                    popup.setHeaderString( "Your citizen's are growing restless" )
                else :
                    popup.setHeaderString( 'A revolution is brewing!' )
                if( odds >= 70 ) :
                    bodStr += "\n\n(Your advisors recommend accepting the terms)"
                elif( odds <= 30 ) :
                    bodStr += "\n\n(Your advisors recommend rejecting the terms)"
                else :
                    bodStr += "\n\n(Your advisors are neutral on these terms)"
                popup.setBodyString( bodStr )
                popup.addButton( 'Accept Terms' )
                buttons = ('accept',)
                popup.addButton( 'Reject Terms' )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    popup.addButton( 'Pay the %d'%(buyOffCost) )
                    buttons += ('buyoff',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on civics")

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: switch civics') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)
                    
                else :
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                    
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        elif( revType == 'religion' ) :
            if( newReligion == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no religion specified")
                return

            if( not pPlayer.isStateReligion() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI has no state religion ... this shouldn't happen")
                return

            newCount = pPlayer.getHasReligionCount( newReligion )
            newHoly = pPlayer.hasHolyCity( newReligion )

            stateCount = pPlayer.getHasReligionCount( pPlayer.getStateReligion() )
            stateHoly = pPlayer.hasHolyCity( pPlayer.getStateReligion() )

            if( stateHoly and not newHoly ) :
                odds = 0
            elif( newHoly and not stateHoly ) :
                if( pPlayer.hasTrait( iSpiritual ) ) :
                    # Spiritual player will spread religion pretty quickly
                    if( stateCount > 2*newCount ) :
                        odds = 60
                    else :
                        odds = 100
                else :
                    if( stateCount > 2*newCount ) :
                        odds = 30
                    elif( newCount > stateCount ) :
                        odds = 100
                    else :
                        odds = 70
            else :
                if( stateCount > 2*newCount ) :
                    odds = 0
                elif( newCount > stateCount ) :
                    odds = 70
                else :
                    odds = 30

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on religion change")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost

                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( peaceful ) :
                    popup.setHeaderString( "Your citizen's are growing restless" )
                else :
                    popup.setHeaderString( 'A revolution is brewing!' )
                if( odds >= 70 ) :
                    bodStr += "\n\n(Your advisors recommend accepting the terms)"
                elif( odds <= 30 ) :
                    bodStr += "\n\n(Your advisors recommend rejecting the terms)"
                else :
                    bodStr += "\n\n(Your advisors are neutral on these terms)"
                popup.setBodyString( bodStr )
                popup.addButton( 'Accept Terms' )
                buttons = ('accept',)
                popup.addButton( 'Reject Terms' )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    popup.addButton( 'Pay the %d'%(buyOffCost) )
                    buttons += ('buyoff',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on religion change")

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: switch religion') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

                else :
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                        
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        elif( revType == 'leader' ) :
            if( newLeaderType == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no leader specified")
                return

            if( not newLeaderName == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential new leader: %s"%(newLeaderName))

            # Compute approval rating (formula from demographics info screen)
            if (pPlayer.calculateTotalCityHappiness() > 0) :
                iHappiness = int((1.0 * pPlayer.calculateTotalCityHappiness()) / (pPlayer.calculateTotalCityHappiness() + \
                                    pPlayer.calculateTotalCityUnhappiness()) * 100)
            else :
                iHappiness = 100

            # Other potential factors:
            # Player rank

            if( numRevCities > pPlayer.getNumCities()/2 ) :
                odds = 70
                if( peaceful ) :
                    odds -= 30
                else :
                    if( pTeam.getAtWarCount(True) > 1 ) :
                        odds += 10
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 20
            elif( numRevCities < pPlayer.getNumCities()/4 ) :
                odds = 30
                if( peaceful ) :
                    odds -= 20
                else :
                    if( pTeam.getAtWarCount(True) > 1 ) :
                        odds += 10
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 30
            else :
                odds = 50
                if( peaceful ) :
                    odds -= 30
                else :
                    if( pTeam.getAtWarCount(True) > 1 ) :
                        odds += 10
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 20

            if( isElection ) :
                if( iHappiness > 60 ) :
                    odds += 30
                elif( iHappiness < 50 ) :
                    odds -= 20

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Approval rating is %d"%(iHappiness))

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on leader change")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.iHappiness = iHappiness
                self.odds = odds
                self.buyOffCost = buyOffCost

                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( peaceful ) :
                    popup.setHeaderString( "Your citizen's are growing restless" )
                else :
                    popup.setHeaderString( 'A revolution is brewing!' )
                if( odds >= 70 ) :
                    bodStr += "\n\nYour advisors recommend accepting the terms."
                elif( odds <= 30 ) :
                    bodStr += "\n\nYour advisors recommend rejecting the terms."
                else :
                    bodStr += "\n\nYour advisors are neutral on these terms."
                if( isElection ) :
                    bodStr += "  Your current approval rating is %d, and %d out of %d cities are in revolt."%(iHappiness,numRevCities,pPlayer.getNumCities())
                popup.setBodyString( bodStr )
                popup.addButton( 'Accept Terms' )
                buttons = ('accept',)
                popup.addButton( 'Reject Terms' )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    if( isElection ) :
                        popup.addButton( 'Buy election for %d'%(buyOffCost) )
                        buttons += ('buyelection',)
                    else :
                        popup.addButton( 'Pay the %d'%(buyOffCost) )
                        buttons += ('buyoff',)

                popup.setUserData(buttons)

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on leader change")

                if( game.getSorenRandNum(100,'Revolt: switch leader') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, iHappiness = iHappiness, buyOffCost = buyOffCost)

                else:
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                        
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        elif( revType == 'independence' ) :

            # Warlords: ask to become vassal state

            if( numRevCities > pPlayer.getNumCities()/2 ) :
                odds = 30
                if( peaceful ) :
                    odds -= 30
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 10
                if( pTeam.getAtWarCount(True) > 1 ) :
                    odds += 10
                if( not joinPlayer == None ) :
                    if( pTeam.isAtWar(joinPlayer.getTeam()) ) :
                        if( peaceful ) :
                            odds -= 40
                        else :
                            odds -= 20
            elif( numRevCities < pPlayer.getNumCities()/4 ) :
                if( numRevCities == 1 ) :
                    odds = 20
                else :
                    odds = 40
                if( peaceful ) :
                    odds -= 20
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 10
                if( not cityList[0].area().getID() == pPlayer.getCapitalCity().area().getID() ) :
                    odds += 20
                if( pTeam.getAtWarCount(True) > 1 ) :
                    odds += 20
                if( not joinPlayer == None ) :
                    if( pTeam.isAtWar(joinPlayer.getTeam()) ) :
                        if( peaceful ) :
                            odds -= 40
                        else :
                            odds -= 30
                if( cityList[0].getNumRevolts(pPlayer.getID()) > 2 ) :
                    odds += 10
            else :
                odds = 30
                if( peaceful ) :
                    odds -= 20
                if( pPlayer.hasTrait(iAggressive) ) :
                    odds -= 10
                if( not cityList[0].area().getID() == pPlayer.getCapitalCity().area().getID() ) :
                    odds += 20
                if( pTeam.getAtWarCount(True) > 1 ) :
                    odds += 10
                if( not joinPlayer == None ) :
                    if( pTeam.isAtWar(joinPlayer.getTeam()) ) :
                        if( peaceful ) :
                            odds -= 30
                        else :
                            odds -= 20
                if( cityList[0].getNumRevolts(pPlayer.getID()) > 2 ) :
                    odds += 10

            if( self.showHumanPopup(pPlayer.getID()) ) :
                # Offer choice
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Offering human choice on independence")

                self.cityList = cityList
                self.pPlayer = pPlayer
                self.revType = revType
                self.peaceful = peaceful
                self.pRevPlayer = pRevPlayer
                self.isJoinWar = isJoinWar
                self.joinPlayer = joinPlayer
                self.newLeaderType = newLeaderType
                self.newLeaderName = newLeaderName
                self.isElection = isElection
                self.newCivic = newCivic
                self.newReligion = newReligion
                self.odds = odds
                self.buyOffCost = buyOffCost

                popup = PyPopup.PyPopup( self.revolutionPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                if( peaceful ) :
                    popup.setHeaderString( "Your citizen's are growing restless" )
                else :
                    popup.setHeaderString( 'A revolution is brewing!' )
                if( odds >= 70 ) :
                    bodStr += "\n\n(Your advisors recommend accepting the terms)"
                elif( odds <= 30 ) :
                    bodStr += "\n\n(Your advisors recommend rejecting the terms)"
                else :
                    bodStr += "\n\n(Your advisors are neutral on these terms)"
                
                popup.setBodyString( bodStr )
                popup.addButton( 'Accept Terms' )
                buttons = ('accept',)
                popup.addButton( 'Reject Terms' )
                buttons += ('reject',)
                if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                    popup.addButton( 'Pay the %d'%(buyOffCost) )
                    buttons += ('buyoff',)
                if( peaceful and joinPlayer == None and (pRevPlayer.getNumCities() == 0) ) :
                    popup.addButton( 'Accept and take control of new civ' )
                    buttons += ('control',)
                elif( not peaceful and joinPlayer == None and (pRevPlayer.getNumCities() == 0) ) :
                    popup.addButton( 'Accept and take control of new civ' )
                    buttons += ('control',)

                popup.setUserData( buttons )

                # Center camera on city
                CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                
                popup.launch(bCreateOkButton = False)

            else :
                # Make AI decision
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Making AI choice on independence")
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Odds are %d"%(odds))

                if( game.getSorenRandNum(100,'Revolt: give independence') < odds ) :
                    self.processRevolution( cityList, pPlayer, revType, peaceful, True, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

                else:
                    if( buyOffCost > 0 and buyOffCost <= pPlayer.getGold() ) :
                        if( peaceful ) :
                            base = .8
                        else :
                            base = .7
                        buyOdds = int( odds/2 + 75*(1.0 - pow(base,pPlayer.getGold()/buyOffCost)) )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyoff odds are %d, with base %f, from cost %d and gold %d"%(buyOdds,base,buyOffCost,pPlayer.getGold()))
                        if( buyOdds > game.getSorenRandNum( 100, 'Rev - AI buyoff decision' ) ) :
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - AI buyingoff rebels")
                            self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion, buyOffCost = buyOffCost, didBuyOff = True)
                            return
                        
                    self.processRevolution( cityList, pPlayer, revType, peaceful, False, pRevPlayer, isJoinWar, joinPlayer, newLeaderType, newLeaderName, isElection, newCivic, newReligion)

        else :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unknown revolution type %s"%(revType))

    def revolutionPopupHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Handling revolution popup")

        newHumanPlayer = None
        termsAccepted = False
        didBuyOff = False
        buttonLabel = netUserData[popupReturn.getButtonClicked()]
        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Button %d pressed, label: %s"%(popupReturn.getButtonClicked(),buttonLabel))

        if( buttonLabel == 'accept' ) :
            termsAccepted = True
        elif( buttonLabel == 'reject' ) :
            termsAccepted = False
        elif( buttonLabel == 'buyoff' ) :
            # Pay the cities not to revolt
            termsAccepted = False
            didBuyOff = True
        elif( buttonLabel == 'control' ) :
            # Give control of new civ
            termsAccepted = True
            newHumanPlayer = self.pRevPlayer.getID()
        elif( buttonLabel == 'buyelection' ) :
            # Attempt buyoff of election results
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Feature not fully implemented %s"%(buttonLabel))
            didBuyOff = True
            termsAccepted = False
        else :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error! Unrecognized button label %s"%(buttonLabel))
            termsAccepted = False

        self.processRevolution( self.cityList, self.pPlayer, self.revType, self.peaceful, termsAccepted, self.pRevPlayer, self.isJoinWar, self.joinPlayer, self.newLeaderType, self.newLeaderName, self.isElection, self.newCivic, self.newReligion, self.iHappiness, self.buyOffCost, didBuyOff )

        if( not newHumanPlayer == None ) :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Human being given control of new player %d"%(newHumanPlayer))
            RevUtils.changeHuman( newHumanPlayer, iPlayerID )

        self.cityList = None
        self.pPlayer = None
        self.revType = None
        self.peaceful = None
        self.pRevPlayer = None
        self.isJoinWar = None
        self.joinPlayer = None
        self.newLeaderType = None
        self.newLeaderName = None
        self.isElection = None
        self.newCivic = None
        self.newReligion = None
        self.iHappiness = None
        self.odds = None
        self.buyOffCost = None 

    def processRevolution( self, cityList, pPlayer, revType, peaceful, termsAccepted = False, pRevPlayer = None, isJoinWar = False, joinPlayer = None, newLeaderType = None, newLeaderName = None, isElection = True, newCivic = None, newReligion = None, iHappiness = None, buyOffCost = -1, didBuyOff = False ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Processing revolution")
        if( self.LOG_DEBUG ) : CyInterface().addImmediateMessage('Processing revolution!!!',"")

        pTeam = gc.getTeam( pPlayer.getTeam() )

        iAggressive = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
        iSpiritual = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_SPIRITUAL')
        numRevCities = len(cityList)
        capital = pPlayer.getCapitalCity()
        capitalArea = capital.area().getID()

        for pCity in cityList :
            pCity.changeNumRevolts( pPlayer.getID(), 1 )
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s has now revolted %d times"%(pCity.getName(),pCity.getNumRevolts(pPlayer.getID())))

        if( didBuyOff ) :
            if( revType == 'leader' and isElection ) :
                # TODO something special for this
                pass
            pPlayer.changeGold( -buyOffCost )
            for pCity in cityList :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Buying off revolutionaries in %s"%(pCity.getName()))
                revIdx = RevUtils.getCityRevolutionIndex( pCity )
                RevUtils.setCityRevolutionIndex( pCity, min([int(self.revReadyFrac*self.revInstigatorThreshold),revIdx-200]) )
                RevUtils.setCityRevolutionCounter( pCity, self.buyoffTurns )
                pCity.changeNumRevolts( pPlayer.getID(), -1 )
            return

        if( revType == 'civics' ) :
            if( newCivic == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no civic specified")
                return

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to switch, lowering rev indices")
                # Do switch
                newCivicOption = gc.getCivicInfo( newCivic ).getCivicOptionType()
                pPlayer.setCivics( newCivicOption, newCivic )
                pPlayer.changeAnarchyTurns( 1 )
                # TODO: figure out way to not allow immeadiate switch back

                # Make those cities happier
                for pCity in cityList :
                    revIdx = RevUtils.getCityRevolutionIndex( pCity )
                    RevUtils.setCityRevolutionIndex( pCity, min([revIdx/2,600]) )
                    RevUtils.setCityRevolutionCounter( pCity, self.acceptedTurns )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player declines to switch")
                if( peaceful ) :
                                    
                    revIdxInc = 250
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                    # Make those cities more unhappy
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityRevolutionIndex( pCity, revIdxInc + 50 )
                        else :
                            RevUtils.changeCityRevolutionIndex( pCity, revIdxInc )
                        RevUtils.setCityRevolutionCounter( pCity, self.deniedTurns )

                else :
                    if( isJoinWar and not joinPlayer == None ) :
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                    else :
                        self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

        elif( revType == 'religion' ) :
            if( newReligion == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no religion specified")
                return

            if( not pPlayer.isStateReligion() ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player has no state religion ... this shouldn't happen")
                return

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to switch, lowering rev indices")
                # Do switch
                pPlayer.convert(newReligion)
                # Make those cities happier
                for pCity in cityList :
                    revIdx = RevUtils.getCityRevolutionIndex( pCity )
                    RevUtils.setCityRevolutionIndex( pCity, min([revIdx/2,600]) )
                    RevUtils.setCityRevolutionCounter( pCity, self.acceptedTurns )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player declines to switch")
                if( not peaceful ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error ... shouldn't have violent request for religion switch")
                    
                revIdxInc = 250
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                # Make those cities more unhappy
                for pCity in cityList :
                    if( not pCity.area().getID() == capitalArea ) :
                        RevUtils.changeCityRevolutionIndex( pCity, revIdxInc + 50 )
                    else :
                        RevUtils.changeCityRevolutionIndex( pCity, revIdxInc )
                    RevUtils.setCityRevolutionCounter( pCity, self.deniedTurns )                        

        elif( revType == 'leader' ) :
            if( newLeaderType == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error, no leader specified")
                return

            if( not newLeaderName == None ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Potential new leader: %s"%(newLeaderName))

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to cede power, lowering rev indices")

                # Lower indices
                for pCity in cityList :
                    revIdx = RevUtils.getCityRevolutionIndex( pCity )
                    RevUtils.setCityRevolutionIndex( pCity, min([revIdx/2,600]) )
                    RevUtils.setCityRevolutionCounter( pCity, self.acceptedTurns )
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

                # Do switch leader
                if( isElection ) :
                    if( iHappiness == None ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!  Happiness not set")
                        iHappiness = 50
                    
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Running an election for a new leader!")
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Approval rating is %d"%(iHappiness))

                    if( numRevCities > (3*pPlayer.getNumCities())/5 ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election lost due to num rev cities")
                        # Continue to change leader code below
                    elif( iHappiness < 45 ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election lost due to low approval rating")
                    elif( game.getSorenRandNum(35,'Revolution: election results') + 30 > iHappiness ) :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election lost by probability")
                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Election won!  Leader stays in power!")
                        if( self.showHumanPopup(pPlayer.getID()) ) :
                            # Report election victory
                            popup = PyPopup.PyPopup()
                            bodStr = 'Victory!  You have won the election!'
                            bodStr += '\n\nThe revolting cities accept the national decision and settle down for the time being.'
                            bodStr += "  However, it will still be prudent to increase their happiness so you don't face this again soon."
                            popup.setBodyString(bodStr)
                            popup.launch()
                        return

                if( self.showHumanPopup(pPlayer.getID()) ) :
                    popup = PyPopup.PyPopup(self.controlLostPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL)
                    gameSpeedMod = (661.0)/(game.getMaxTurns() + 1.0)
                    if( gameSpeedMod == 661.0 ) : gameSpeedMod = 1.0
                    if( isElection ) :
                        iNumTurns = int( gameSpeedMod*15 )
                        bodStr = 'Terrible news!  You have lost the election and control over your civ!'
                    else :
                        iNumTurns = int( gameSpeedMod*24 )
                        bodStr = 'Your rule is over and a new leader assumes the throne!'

                    bodStr += '\n\nHowever, control will be returned to you in %d turns'%(iNumTurns)
                    popup.setBodyString( bodStr )
                    popup.setUserData( (iNumTurns,) )
                    popup.launch()

                    #game.setAIAutoPlay( iNumTurns )

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Changing leader to %s"%(newLeaderName))

                if( newLeaderType == pPlayer.getLeaderType() ) :
                    # Just change the name of leader
                    #pPlayer.setName( newLeaderName )
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No leader change, leader's of same type")
                else :
                    if( pPlayer.getID() == game.getActivePlayer() ) :
                        # Don't need to save this as player can't exit while in automation
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Skipping leader change for human")
                        #self.humanLeaderType = pPlayer.getLeaderType()
                    else :
                        RevUtils.changeCiv(pPlayer.getID(),pPlayer.getCivilizationType(),newLeaderType)

                iSpiritual = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_SPIRITUAL')
                if( pPlayer.hasTrait(iSpiritual) ) :
                    pPlayer.changeAnarchyTurns(1)

            else:
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player clings to power")
                
                if( peaceful ) :
                    revIdxInc = 250
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityRevolutionIndex( pCity, revIdxInc + 50 )
                        else :
                            RevUtils.changeCityRevolutionIndex( pCity, revIdxInc )
                        RevUtils.setCityRevolutionCounter( pCity, self.deniedTurns ) 

                else :
                    if( isJoinWar and not joinPlayer == None ) :
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                    else :
                        self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

        elif( revType == 'independence' ) :

            if( termsAccepted ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player opts to give independence, lowering rev indices")
                
                # Lower indices (makes life a little easier for new civ ...)
                for pCity in cityList :
                    revIdx = RevUtils.getCityRevolutionIndex( pCity )
                    RevUtils.setCityRevolutionIndex( pCity, min([revIdx/2,600]) )
                    # Other changes handled by acquired city logic
                    if( pCity.getNumRevolts(pPlayer.getID()) > 1 ) :
                        pCity.changeNumRevolts( pPlayer.getID(), -2 )
                    else :
                        pCity.changeNumRevolts( pPlayer.getID(), -1 )

                if( joinPlayer == None ) :
                    # Grant independence
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are taking over the revolutionary cities"%(pRevPlayer.getCivilizationDescription(0)))
                    bIsBarbRev = pRevPlayer.isBarbarian()
                    
                    if( not bIsBarbRev ) :
                        pRevTeam = gc.getTeam( pRevPlayer.getTeam() )
                        if( not pRevTeam.isAlive() ) :
                            if( pRevTeam.isAtWar(pPlayer.getTeam()) ) :
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Reincarnated %s ending war with %s"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0)))
                                pRevTeam.makePeace( pPlayer.getTeam() )

                        pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), 3 )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))

                    # Since instigator is first in list, it will become capital
                    for pCity in cityList :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquiring %s, pop %d"%(pCity.getName(),pCity.getPopulation()))
                        # Move units out of city?  Handled by trade style
                        [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, pRevPlayer )

                        RevUtils.setCityRevolutionCiv( pCity, -1 )

                        # Acquire city
##                        if( pRevTeam.isAtWar( pPlayer.getTeam() ) ) :
##                            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s acquired using conversion"%(pCity.getName()))
##                            #pRevPlayer.acquireCity( pCity, True, False )
##                            cityPlot = pCity.plot()
##                            pCity.plot().setOwner( pRevPlayer.getID() )
##                            pCity = cityPlot.getPlotCity()
##
##                        else :
                            
                        cityPlot = pCity.plot()
                        pCity.plot().setOwner( pRevPlayer.getID() )
                        pCity = cityPlot.getPlotCity()

                        #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s at %d, %d"%(pCity.getName(),pCity.getX(),pCity.getY()))

                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Culture in %s: %d, plot %d"%(pCity.getName(),pCity.getCulture(pPlayer.getID()),pCity.plot().getCulture(pPlayer.getID())))

                        newCulVal = int( self.revCultureModifier*max([pCity.getCulture(pPlayer.getID()),pCity.countTotalCultureTimes100()/200]) )
                        newPlotVal = int( self.revCultureModifier*max([pCity.plot().getCulture(pPlayer.getID()),pCity.plot().countTotalCulture()/2]) )
                        RevUtils.giveCityCulture( pCity, pRevPlayer.getID(), newCulVal, newPlotVal, overwriteHigher = False )

                        # Give a few defenders
                        ix = pCity.getX()
                        iy = pCity.getY()
                        if( not iBestDefender == UnitTypes.NO_UNIT ) : pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        #if( not iBestDefender == UnitTypes.NO_UNIT ) : pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        if( not iCounter == UnitTypes.NO_UNIT ) :
                            pRevPlayer.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                        elif( not iBestDefender == UnitTypes.NO_UNIT ) :
                            pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        pRevPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                        if( pRevPlayer.getNumCities() <= 2 ) :
                            # Extra units for first two cities
                            if( not iAttack == UnitTypes.NO_UNIT ) :
                                pRevPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                                pRevPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                            elif( not iBestDefender == UnitTypes.NO_UNIT ) :
                                pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                                pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        elif( gc.getTeam(pPlayer.getTeam()).isAtWar(pRevPlayer.getTeam()) ) :
                            if( not iBestDefender == UnitTypes.NO_UNIT ) :
                                pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                                #pRevPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )

                        if( not bIsBarbRev ) : RevUtils.setCityRevolutionCiv( pCity, pRevPlayer.getID() )

                else :
                    # Join cultural owner
                    if( self.showHumanPopup(pPlayer.getID()) ) :
                        # Both tests needed for AutoPlay and multiplayer

                        self.cityList = cityList
                        self.pPlayer = pPlayer
                        self.revType = revType
                        self.peaceful = peaceful
                        self.pRevPlayer = pRevPlayer
                        self.isJoinWar = isJoinWar
                        self.joinPlayer = joinPlayer
                        self.newLeaderType = newLeaderType
                        self.newLeaderName = newLeaderName
                        self.isElection = isElection
                        self.newCivic = newCivic
                        self.newReligion = newReligion
                        self.buyOffCost = buyOffCost

                        # Offer human the option of accepting the cities
                        PyPopup.PyPopup( self.joinHumanPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL, bDynamic = False)
                        
                        popup.setHeaderString( "Good news!" )
                        
                        bodStr = "The citizens of %s"%(cityList[0].getName())
                        if( len(cityList) > 1 ) :
                            for city in cityList[1:len(cityList)-1]:
                                bodStr += ', %s'%(city.getName())
                            bodStr += ', and %s'%(cityList[-1].getName())

                        bodStr += ' have been granted the freedom to rejoin your empire by %s!'%(pPlayer.getName())
                        bodStr += '  You can regain your lost territory without firing a shot!'
                        bodStr += '\n\nIf you do not want them back, the revolutionaries will instead form the %s.'%(pRevPlayer.getCivilizationDescription(0))
                        bodStr += '  What shall we tell the revolutionaries?'
                        
                        popup.setBodyString( bodStr )
                        popup.addButton( 'Welcome back' )
                        buttons = ('welcome')
                        popup.addButton( "You're not welcome here" )
                        buttons += ('goaway',)

                        # Center camera on city
                        CyCamera().JustLookAt( cityList[0].plot().getPoint() )
                        
                        popup.launch(bCreateOkButton = False)

                    else :
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s are claiming their cultural cities"%(joinPlayer.getCivilizationDescription(0)))

                        bIsBarbRev = joinPlayer.isBarbarian()

                        if( not bIsBarbRev ) :
                            # Improve relations
                            joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), 2 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                        
                        for pCity in cityList :
                            # Move units out of city?  Handled by trade style?
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquiring %s, pop %d"%(pCity.getName(),pCity.getPopulation()))

                            [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, joinPlayer )

                            RevUtils.setCityRevolutionCiv( pCity, -1 )

                            # Acquire city
                            #joinPlayer.acquireCity( pCity, False, True )
                            cityPlot = pCity.plot()
                            pCity.plot().setOwner( joinPlayer.getID() )
                            pCity = cityPlot.getPlotCity()

                            # Give a few defenders
                            ix = pCity.getX()
                            iy = pCity.getY()
                            if( not iBestDefender == UnitTypes.NO_UNIT ) : joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            #if( not iBestDefender == UnitTypes.NO_UNIT ) : joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            if( not iCounter == UnitTypes.NO_UNIT ) :
                                joinPlayer.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                            elif( not iBestDefender == UnitTypes.NO_UNIT ) :
                                joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            if( joinPlayer.getNumCities() <= 2 ) :
                                # Extra units for first two cities (unlikely in this case)
                                joinPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                                if( not iAttack == UnitTypes.NO_UNIT ) :
                                    joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                                    joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                                elif( not iBestDefender == UnitTypes.NO_UNIT ) :
                                    joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                                    joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                            elif( gc.getTeam(pPlayer.getTeam()).isAtWar(joinPlayer.getTeam()) ) :
                                if( not iBestDefender == UnitTypes.NO_UNIT ) :
                                    joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                                    #joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )

                            if( not bIsBarbRev ) : RevUtils.setCityRevolutionCiv( pCity, joinPlayer.getID() )

            else:
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Player refuses to let civ walk")
                
                if( peaceful ) :
                    # Cities get more pissed off
                    revIdxInc = 250
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Peaceful, increasing rev indices")
                    for pCity in cityList :
                        if( not pCity.area().getID() == capitalArea ) :
                            RevUtils.changeCityRevolutionIndex( pCity, revIdxInc + 50 )
                        else :
                            RevUtils.changeCityRevolutionIndex( pCity, revIdxInc )
                        RevUtils.setCityRevolutionCounter( pCity, self.deniedTurns )

                    if( not joinPlayer == None ) :
                        joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -2 )
                        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))

                else :
                    # Violent uprising!!!
                    if( isJoinWar and not joinPlayer == None ) :
                        self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, isJoinWar )
                    else :
                        if( not joinPlayer == None ) :
                            joinPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -3 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(joinPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),joinPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                            
                            # Check if joinPlayer would like to declare war on player
                            warOdds = RevUtils.computeWarOdds( joinPlayer, pPlayer, cityList[0].area() )

                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - War odds are %d"%(warOdds))

                            if( warOdds > 25 and warOdds > game.getSorenRandNum(100,'Revolution: war') ) :
                                # have joinPlayer declare war on player
                                joinTeam = gc.getTeam( joinPlayer.getTeam() )
                                joinTeam.declareWar(pPlayer.getID(),True)
                                isJoinWar = True

                                self.spawnRevolutionaries( cityList, pPlayer, joinPlayer, True )

                            else :
                                pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                                pRevPlayer.AI_changeAttitudeExtra( joinPlayer.getID(), 5 )
                                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                                pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )

                                self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

                        else :
                            pRevPlayer.AI_changeAttitudeExtra( pPlayer.getID(), -5 )
                            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(pRevPlayer.getCivilizationDescription(0),pPlayer.getCivilizationDescription(0),pRevPlayer.AI_getAttitudeExtra(pPlayer.getID())))
                            pPlayer.AI_changeAttitudeExtra( pRevPlayer.getID(), -3 )
                            
                            self.spawnRevolutionaries( cityList, pPlayer, pRevPlayer, isJoinWar )

        else :
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unknown revolution type %s"%(revType))

        return

    def controlLostHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Handling human lost control of civ popup")

        iNumTurns = netUserData[0]
        game.setAIAutoPlay( iNumTurns )
                            

    def joinHumanHandler( self, iPlayerID, netUserData, popupReturn ) :

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Handling join human popup")

        buttonLabel = netUserData[popupReturn.getButtonClicked()]

        if( buttonLabel == 'welcome' ) :
            # Welcome back!
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolutionaries welcomed with open arms")

            self.joinPlayer.AI_changeAttitudeExtra( self.pPlayer.getID(), 2 )
            #if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - %s's Extra Attitude towards %s now %d"%(self.joinPlayer.getCivilizationDescription(0),self.pPlayer.getCivilizationDescription(0),self.joinPlayer.AI_getAttitudeExtra(self.pPlayer.getID())))
            
            for pCity in self.cityList :
                # Move units out of city?  Handled by trade style?
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Acquiring %s"%(pCity.getName()))

                [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, self.joinPlayer )

                RevUtils.setCityRevolutionCiv( pCity, -1 )

                # Acquire city
                #joinPlayer.acquireCity( pCity, False, True )
                cityPlot = pCity.plot()
                pCity.plot().setOwner( self.joinPlayer.getID() )
                pCity = cityPlot.getPlotCity()

                # Give a few defenders
                ix = pCity.getX()
                iy = pCity.getY()
                if( not iBestDefender == UnitTypes.NO_UNIT ) : self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                #if( not iBestDefender == UnitTypes.NO_UNIT ) : self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                if( not iCounter == UnitTypes.NO_UNIT ) :
                    self.joinPlayer.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                elif( not iBestDefender == UnitTypes.NO_UNIT ) :
                    self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                if( self.joinPlayer.getNumCities() <= 2 ) :
                    # Extra units for first two cities (unlikely in this case)
                    self.joinPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                    if( not iAttack == UnitTypes.NO_UNIT ) :
                        self.joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                        self.joinPlayer.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                    elif( not iBestDefender == UnitTypes.NO_UNIT ) :
                        self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                elif( gc.getTeam(self.pPlayer.getTeam()).isAtWar(self.joinPlayer.getTeam()) ) :
                    if( not iBestDefender == UnitTypes.NO_UNIT ) :
                        self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                        #self.joinPlayer.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )

                RevUtils.setCityRevolutionCiv( pCity, self.joinPlayer.getID() )

        elif( buttonLabel == 'goaway' ) :
            # Go away!
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revolutionaries rebuffed, striking out on their own")

            self.processRevolution( self.cityList, self.pPlayer, self.revType, self.peaceful, True, self.pRevPlayer, False, None, self.newLeaderType, self.newLeaderName, self.isElection, self.newCivic, self.newReligion, self.iHappiness, self.buyOffCost )

        else :
            # WTF?
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Unexpected button label %s"%(buttonLabel))

        self.cityList = None
        self.pPlayer = None
        self.revType = None
        self.peaceful = None
        self.pRevPlayer = None
        self.isJoinWar = None
        self.joinPlayer = None
        self.newLeaderType = None
        self.newLeaderName = None
        self.isElection = None
        self.newCivic = None
        self.newReligion = None
        self.iHappiness = None
        self.odds = None
        self.buyOffCost = None


    def spawnRevolutionaries( self, cityList, pPlayer, pRevPlayer, isJoinWar = False ) :

        if( self.LOG_DEBUG ) :
            if( pPlayer.isBarbarian() ) :
                print "Error: spawning revs for barb player"

        bIsBarbRev = pRevPlayer.isBarbarian()

        if( gc.getPlayer(game.getActivePlayer()).canContact(pPlayer.getID()) ) :
            if( bIsBarbRev ) :
                if( game.getActivePlayer() == pPlayer.getID() ) :
                    mess = "Disorganized rebels have risen up against you!"
                else :
                    mess = "A violent revolution has occured in the %s!!!"%(PyPlayer(pPlayer.getID()).getCivilizationName())
                    mess += "  Disorganized rebels have risen up as the region descends into civil war!"
            else :
                if( game.getActivePlayer() == pPlayer.getID() ) :
                    if( pRevPlayer.isAlive() ) :
                        mess = "The revolutionionaries join the %s in their war against you!"%(pRevPlayer.getCivilizationDescription(0))
                    else :
                        mess = "The %s have risen up against you in revolution!"%(pRevPlayer.getCivilizationDescription(0)) 
                else :
                    mess = "A violent revolution has occured in the %s!!!"%(PyPlayer(pPlayer.getID()).getCivilizationName())
                    if( pRevPlayer.isAlive() ) :
                        mess += "  The revolutionaries join the %s as war engulfs the region."%(pRevPlayer.getCivilizationDescription(0))
                    else :
                        mess += "  The %s have risen up as the region descends into civil war!"%(pRevPlayer.getCivilizationDescription(0))
            CyInterface().addImmediateMessage(mess,"")

        if( not bIsBarbRev ) :
            pRevTeam = gc.getTeam( pRevPlayer.getTeam() )
            if( not pRevTeam.isAtWar(pPlayer.getTeam()) ) :
                pRevTeam.declareWar( pPlayer.getTeam(), True )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s revolutionaries declare war on the %s!"%(pRevPlayer.getCivilizationAdjective(0),pPlayer.getCivilizationDescription(0)))
            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - The %s revolutionaries join in the war against the %s!"%(pRevPlayer.getCivilizationAdjective(0),pPlayer.getCivilizationDescription(0)))

            if( pRevPlayer.getGold() < 50 ) :
                pRevPlayer.changeGold( 30 + game.getSorenRandNum(35*len(cityList),'Revolt: give gold') )
            else :
                pRevPlayer.changeGold( 10 + game.getSorenRandNum(15*len(cityList),'Revolt: give gold') )

        if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %s revolutionaries!!!"%(pRevPlayer.getCivilizationAdjective(0)))

        # Additional ideas:
        # Give a Great General in some cases?
        # Who should the revs be able to contact?
        # Give spies to later revs?

        iGoodyMap = CvUtil.findInfoTypeNum(gc.getGoodyInfo,gc.getNumGoodyInfos(),'GOODY_MAP')
        revIdxInc = 300
        
        for [cityIdx,pCity] in enumerate(cityList) :

            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - In %s, with rev idx %d (%d local)"%(pCity.getName(),RevUtils.getCityRevolutionIndex( pCity ),RevUtils.getCityLocalRevIndex( pCity )))

            spawnableUnits = RevUtils.getUprisingUnitTypes( pCity, pRevPlayer, isJoinWar )
            [iWorker,iBestDefender,iCounter,iAttack] = RevUtils.getHandoverUnitTypes( pCity, pRevPlayer )

            if( len(spawnableUnits) < 1 ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - ERROR!!! No rev units possible in %s"%(pCity.getName()))
                continue

            revIdx = RevUtils.getCityRevolutionIndex( pCity )
            ix = pCity.getX()
            iy = pCity.getY()

            spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 2, iSpawnPlotOwner = pRevPlayer.getID(), bCheckForEnemy = True )
            if( len(spawnablePlots) == 0 ) :
                spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 3, iSpawnPlotOwner = pRevPlayer.getID(), bCheckForEnemy = True )
            if( len(spawnablePlots) == 0 ) :
                spawnablePlots = RevUtils.getSpawnablePlots( ix, iy, pRevPlayer, bLand = True, bIncludePlot = False, bIncludeCities = False, bSameArea = True, iRange = 3, iSpawnPlotOwner = -1, bCheckForEnemy = False )

            revSpawnLoc = None
            if( len(spawnablePlots) > 0 ) :
                revSpawnLoc = spawnablePlots[game.getSorenRandNum(len(spawnablePlots),'Revolution: Pick rev plot')]

            iNumUnits = int( self.strengthModifier*(1.8*revIdx/(1.0*self.revReadyFrac*self.revInstigatorThreshold) + 1) + .5 )
            if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Initial enlistment: %d"%(iNumUnits))
            if( cityIdx == 0 ) :
                # More for instigator
                iNumUnits += 2
            if( pRevPlayer.getCivilizationType() == RevUtils.getCityRevolutionCiv( pCity ) ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Repeat revolution, increasing enlistment")
                iNumUnits += 1 + game.getSorenRandNum( 3, 'Revolution: more units' )
            if( RevUtils.getCityLocalRevIndex( pCity ) < 0 and iNumUnits > 4 ) :
                iNumUnits = 2 + iNumUnits/2
            elif( RevUtils.getCityLocalRevIndex( pCity ) > 0 ) :
                iNumUnits += 1
                
            if( isJoinWar ) :
                # Joining established player in war
                iNumUnits = min( [iNumUnits, pCity.getPopulation()/3] )
                # Really small cities shouldn't be revolting, but just in case
                iNumUnits = max( [iNumUnits, 3] )
            elif( bIsBarbRev ) :
                # Lower max number of barb units
                iNumUnits = min( [iNumUnits - 2, pCity.getPopulation()/2] )
                iNumUnits = max( [iNumUnits, 2] )
            else :
                iNumUnits = min( [iNumUnits, pCity.getPopulation()] )
                iNumUnits = max( [iNumUnits, 3] ) 
            
            revControl = False
            # Determine who controls city, revs or player
            
            if( revSpawnLoc == None ) :
                # If no plot on which to spawn revs, they get city and owners units flee
                # What about isJoinWar case?
                revControl = True
            else :
                # Compare strength of revolution and garrison
                iRevStrength = iNumUnits
                if( (pCity.unhappyLevel(0) - pCity.happyLevel()) > 0 ) :
                    iRevStrength += 2
                if( isJoinWar ) :
                    iRevStrength -= 2
                if( bIsBarbRev ) :
                    iRevStrength -= 4

                iGarrisonStrength = pCity.plot().getNumDefenders(pPlayer.getID()) + 1
                iGarrisonStrength = int( iGarrisonStrength*(110 + pCity.getBuildingDefense())/100.0 )

                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Rev strength: %d,  Garrison strength: %d"%(iRevStrength,iGarrisonStrength))

                if( iRevStrength > iGarrisonStrength ) :
                    # Revolutionaries out muscle the city garrison and take control
                    revControl = True
                else :
                    # Spawn in countryside
                    revControl = False
            
            if( revControl ) :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Revs take control of %s (%d,%d)"%(pCity.getName(),pCity.getX(),pCity.getY()))
            
                # Turn off rebellious city capture logic, all components handled here
                RevUtils.setCityRevolutionCiv( pCity, -1 )
                
                # Run wounded soldiers out of Town
                iInjureMax = 50
                retreatPlots = RevUtils.getSpawnablePlots( ix, iy, pPlayer, bLand = True, bIncludePlot = False, bIncludeCities = True, bSameArea = True, iRange = 3, iSpawnPlotOwner = pPlayer.getID(), bCheckForEnemy = True )
                if( len(retreatPlots) == 0 ) :
                    iInjureMax = 60
                    retreatPlots = RevUtils.getSpawnablePlots( ix, iy, pPlayer, bLand = True, bIncludePlot = False, bIncludeCities = True, bSameArea = False, iRange = 4, iSpawnPlotOwner = -1, bCheckForEnemy = True )
                if( len(retreatPlots) == 0 ) :
                    iInjureMax = 70
                    # Try to move to another of players cities
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - No nearby plots, trying move to another of players cities")
                    for otherCity in PyPlayer( pPlayer.getID() ).getCityList() :
                        pOtherCity = otherCity.GetCy()
                        if( not pOtherCity.getID() == pCity.getID() ) :
                            retreatPlots.append([pOtherCity.getX(),pOtherCity.getY()])
                            
                if( len(retreatPlots) > 0 ) :
                    retreatLoc = retreatPlots[game.getSorenRandNum(len(retreatPlots),'Revolution: Pick rev plot')]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City garrison retreating to %d,%d"%(retreatLoc[0], retreatLoc[1]))
                else :
                    retreatLoc = None
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City garrison massacred (had nowhere to go)")
                    # pPlayer is about to die anyway

                # Defections?

                if( not retreatLoc == None ) :
                    RevUtils.moveEnemyUnits( ix, iy, pRevPlayer.getID(), retreatLoc[0], retreatLoc[1], iInjureMax = iInjureMax, bDestroyNonLand = True )

                # Acquire city by cultural conversion
                #pRevPlayer.acquireCity( pCity, False, False )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Population of %s before is %d"%(pCity.getName(),pCity.getPopulation()))
                cityPlot = pCity.plot()
                pCity.plot().setOwner( pRevPlayer.getID() )
                pCity = cityPlot.getPlotCity()
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Population of %s after is %d"%(pCity.getName(),pCity.getPopulation()))

                if( pCity.getPopulation() < 1 ) :
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Error!  City %s is empty"%(pCity.getName()))

                newCulVal = int( self.revCultureModifier*max([pCity.getCulture(pPlayer.getID()),pCity.countTotalCultureTimes100()/200]) )
                newPlotVal = int( self.revCultureModifier*max([pCity.plot().getCulture(pPlayer.getID()),pCity.plot().countTotalCulture()/2]) )
                RevUtils.giveCityCulture( pCity, pRevPlayer.getID(), newCulVal, newPlotVal, overwriteHigher = False )

                # Spawn revs in the city, plus an extra defender or two
                # Couple revs die in taking control of city:
                if( iNumUnits > 9 ) :
                    iNumUnits -= 2
                elif( iNumUnits > 5 ) :
                    iNumUnits -= 1
                    
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %d units for city of size %d"%(iNumUnits,pCity.getPopulation()))

                newUnitList = list()
                for i in range(0,iNumUnits) :
                    newUnitID = spawnableUnits[game.getSorenRandNum( len(spawnableUnits), 'Revolution: pick unit' )]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - In %s, spawning %s"%(pCity.getName(),PyInfo.UnitInfo(newUnitID).getDescription()))
                    newUnit = pRevPlayer.initUnit( newUnitID, ix, iy, UnitAITypes.NO_UNITAI )
                    newUnitList.append( newUnit )
                
                # Give a few extra defenders and a worker
                if( revIdx > int(self.revInstigatorThreshold) ) :
                    if( not iBestDefender == UnitTypes.NO_UNIT ) :
                        newUnitList.append( pRevPlayer.initUnit(iBestDefender, ix, iy, UnitAITypes.NO_UNITAI) )
                pRevPlayer.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )

                # Injure units to simulate the lack of training in rebel troops
                for newUnit in newUnitList :
                    iDamage = 10 + game.getSorenRandNum(25,'Rev - Injure unit')
                    newUnit.setDamage( iDamage, pPlayer.getID() )

                # Should buildings stay or some destroyed?

                # Reveal surrounding countryside
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Giving map")
                pRevPlayer.receiveGoody( gc.getMap().plot(ix,iy), iGoodyMap, newUnitList[0] )
                pRevPlayer.receiveGoody( gc.getMap().plot(ix,iy), iGoodyMap, newUnitList[0] )

                # No more revolutions for a while
                RevUtils.setCityRevolutionCounter( pCity, self.turnsBetweenRevs )
                #RevUtils.setCityRebellionCounter( pCity, max([RevUtils.getCityRebellionCounter(pCity),self.config.getint("Rebellion", "Turns", 10)/2]) )
                if( not bIsBarbRev ) :
                    RevUtils.setCityRevolutionCiv( pCity, pRevPlayer.getCivilizationType() )
                
            else :
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Owner keeps control of %s (%d,%d), revs spawning at %d,%d"%(pCity.getName(),ix,iy,revSpawnLoc[0],revSpawnLoc[1]))

                # City in disorder
                iTurns = revIdx/int(self.revReadyFrac*self.revInstigatorThreshold)
                if( RevUtils.getCityLocalRevIndex( pCity ) > 4 ) :
                    iTurns = int(iTurns*1.5)
                pCity.setOccupationTimer( min([iTurns,12]) )
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - City occupation timer set to %d"%(pCity.getOccupationTimer()))

                # Wound player's units?
                unitList = RevUtils.getEnemyUnits( ix, iy, pRevPlayer.getID() )
                for pUnit in unitList :
                    pUnit.setDamage( 20 + game.getSorenRandNum(20,'Revolution: Wound units'), pRevPlayer.getID() )

                # Defections?
                    
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Spawning %d units for city of size %d"%(iNumUnits,pCity.getPopulation()))

                # Spawn rev units outside city
                newUnitList = list()
                for i in range(0,iNumUnits) :
                    newUnitID = spawnableUnits[game.getSorenRandNum( len(spawnableUnits), 'Revolution: pick unit' )]
                    if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Outside of %s, spawning %s"%(pCity.getName(),PyInfo.UnitInfo(newUnitID).getDescription()))
                    newUnit = pRevPlayer.initUnit( newUnitID, revSpawnLoc[0], revSpawnLoc[1], UnitAITypes.NO_UNITAI )
                    newUnitList.append( newUnit )

                # Injure units to simulate the lack of training in rebel troops
                for newUnit in newUnitList :
                    iDamage = 10 + game.getSorenRandNum(20,'Rev - Injure unit')
                    newUnit.setDamage( iDamage, pPlayer.getID() )

                # Reveal surrounding countryside
                if( self.LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Giving map")
                pRevPlayer.receiveGoody( gc.getMap().plot(revSpawnLoc[0], revSpawnLoc[1]), iGoodyMap, newUnitList[0] )
                pRevPlayer.receiveGoody( gc.getMap().plot(revSpawnLoc[0], revSpawnLoc[1]), iGoodyMap, newUnitList[0] )
                
                RevUtils.changeCityRevolutionIndex( pCity, revIdxInc )
                RevUtils.setCityRevolutionCounter( pCity, self.turnsBetweenRevs )
                #RevUtils.setCityRebellionCounter( pCity, max([RevUtils.getCityRebellionCounter(pCity),self.config.getint("Rebellion", "Turns", 10)/2]) )
                if( not bIsBarbRev ) : RevUtils.setCityRevolutionCiv( pCity, pRevPlayer.getCivilizationType() )

        return
