# Tester.py
#
# by jdog5000
#
# Provides event debug output

from CvPythonExtensions import *
import sys
import Popup as PyPopup
from PyHelpers import PyPlayer
import CvEventManager
from PyHelpers import *
import CvUtil

# globals
gc = CyGlobalContext()
localText = CyTranslator()
game = CyGame()

class Tester :

    def __init__(self, customEM, config):

        CvUtil.pyPrint("Initializing Tester")

        self.LOG_DEBUG = config.getboolean("CONFIG", "DebugMode", False )
        self.EventKeyDown=6
        self.testerPopup = 8000

        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'greatPersonBorn', self.onGreatPersonBorn )
        #if( self.LOG_DEBUG ) : customEM.addEventHandler( 'techAcquired', self.onTechAcquired )
        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'religionFounded', self.onReligionFounded )
        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'changeWar', self.onChangeWar )
        if( self.LOG_DEBUG ) : customEM.addEventHandler( 'cityLost', self.onCityLost )
        customEM.addEventHandler( 'GameStart', self.onGameStart )
        customEM.addEventHandler( 'OnLoad', self.onGameLoad )
        customEM.addEventHandler( "kbdEvent", self.onKbdEvent )
        #customEM.addEventHandler( "EndPlayerTurn", self.endTurnPopup )

        customEM.setPopupHandler( self.testerPopup, ["testerPopup",self.testerPopupHandler,self.blankHandler] )

        self.customEM = customEM

        self.config = config
        self.foundConfig = config.getboolean("CONFIG", "FoundConfig", False)
        self.showActivePopup = config.getboolean("CONFIG", "ActivePopup", False)

    def blankHandler( self, playerID, netUserData, popupReturn ) :
        # Dummy handler to take the second event for popup
        return

    def onGameStart( self, argsList ) :
        if( self.showActivePopup or not self.foundConfig ) :
            self.onGameLoad( None )

    def onGameLoad( self, argsList ) :

        if( not self.foundConfig ) :
            popup = PyPopup.PyPopup( )       
            bodStr = "WARNING:  Revolution.ini not found!  Revolution components using default settings."
            bodStr += "  Check mod installation directory, should be: (civ install directory)\Warlords\Mods\Revolution\."
            popup.setBodyString( bodStr )
            popup.launch()
            
        if( self.showActivePopup ) :
            bodStr = "Revolution mod running with the following components:\n"
            if( self.config.getboolean("AIAutoPlay", "Enable", True) ) :
                bodStr += "\nAIAutoPlay"
            if( self.config.getboolean("BarbarianCiv", "Enable", True) ) :
                bodStr += "\nBarbarianCiv"
            if( self.config.getboolean("Rebellion", "Enable", True) ) :
                bodStr += "\nRebellion"
            if( self.config.getboolean("ChangePlayer", "Enable", False) ) :
                bodStr += "\nChangePlayer"
            if( self.config.getboolean("Revolution", "Enable", False) ) :
                bodStr += "\nRevolution"

            bodStr += "\n\nMax civs allowed: %d"%(gc.getMAX_CIV_PLAYERS())
            bodStr += "\nTurns in game: %d"%(game.getMaxTurns())

            popup = PyPopup.PyPopup( )       
            popup.setBodyString( bodStr )
            popup.launch()
        

    def onGreatPersonBorn(self, argsList):
        'Unit Promoted'
        pUnit, iPlayer, pCity = argsList
        player = PyPlayer(iPlayer)
        if pUnit.isNone() or pCity.isNone():
                return

        CvUtil.pyPrint('A %s was born for %s in %s' %(pUnit.getName(), player.getCivilizationName(), pCity.getName()))

    
    def onTechAcquired(self, argsList):
		'Tech Acquired'
		iTechType, iTeam, iPlayer, bAnnounce = argsList
		# Note that iPlayer may be NULL (-1) and not a refer to a player object

		CvUtil.pyPrint('%s was finished by Team %d' 
			%(PyInfo.TechnologyInfo(iTechType).getDescription(), iTeam))

    def onReligionFounded(self, argsList):
		'Religion Founded'
		iReligion, iFounder = argsList
		player = PyPlayer(iFounder)
		
		CvUtil.pyPrint('Player %d Civilization %s has founded %s'
			%(iFounder, player.getCivilizationName(), gc.getReligionInfo(iReligion).getDescription()))


    def onChangeWar(self, argsList):
		'War Status Changes'
		bIsWar = argsList[0]
		iPlayer = argsList[1]
		iRivalTeam = argsList[2]
		if (bIsWar):
			strStatus = "declared war"
		else:
			strStatus = "declared peace"
		
		CvUtil.pyPrint('Player %d Civilization %s has %s on Team %d'%(iPlayer, gc.getPlayer(iPlayer).getCivilizationDescription(0), strStatus, iRivalTeam))

    def onCityLost(self, argsList):
		'City Lost'
		city = argsList[0]
		player = PyPlayer(city.getOwner())

		CvUtil.pyPrint('City %s was lost by Player %d Civilization %s' 
			%(city.getName(), player.getID(), player.getCivilizationName()))
		
    def onKbdEvent(self, argsList ):
        'keypress handler'
        eventType,key,mx,my,px,py = argsList

        if ( eventType == self.EventKeyDown ):
	    theKey=int(key)
	    
            # For debug or trial only
##	    if( theKey == int(InputTypes.KB_T) and self.customEM.bShift and self.customEM.bCtrl ) :
##
##                popup = PyPopup.PyPopup(self.testerPopup, contextType = EventContextTypes.EVENTCONTEXT_ALL)
##                popup.setHeaderString( 'Test popup' )
##                popup.setBodyString( 'Where to you want to go today?' )
##                buttons = tuple()
##                popup.addButton( 'First button' )
##                popup.addButton( 'Offer button' )
##                popup.addButton( 'Buyoff button' )
##                popup.setUserData( ('first','offer','buyoff') )
##                popup.launch()

            if( theKey == int(InputTypes.KB_Q) and self.customEM.bShift and self.customEM.bCtrl ) :
                self.showActivePopup = True
                self.onGameLoad( [] )



    def testerPopupHandler( self, iPlayerID, netUserData, popupReturn ) :

        CvUtil.pyPrint('Handling tester popup')

        print netUserData

        print "Attempting to read out pressed button"

        print netUserData[popupReturn.getButtonClicked()]
        popup = PyPopup.PyPopup()
        popup.setBodyString( netUserData[popupReturn.getButtonClicked()] )
        popup.launch()

    def endTurnPopup( self, argsList ) :

        iGameTurn, iPlayer = argsList

        if( iPlayer == game.getActivePlayer() ) :
            popup = PyPopup.PyPopup()
            popup.setBodyString( "Attempting to pause" )
            popup.popup.setTimer( 10000 )

            count = 0
            while( not popup.isNone() ) :
                count += 1
                if( count > 1000 ) :
                    break

            CyInterface().addImmediateMessage("Reached pre-count %d"%(count),"")
                    
            popup.launch()

            count = 0
            while( not popup.isNone() ) :
                count += 1
                if( count > 1000 ) :
                    break

            CyInterface().addImmediateMessage("Reached post-count %d"%(count),"")
        

                
