# Utility functions for Revolution Mod
#
# by jdog5000
# Version 0.65

from CvPythonExtensions import *
import CvUtil
import PyHelpers
import pickle

# globals
gc = CyGlobalContext()
PyPlayer = PyHelpers.PyPlayer
PyInfo = PyHelpers.PyInfo
game = CyGame()

# The City Script array IDs when loading/saving data
iCityWarningCounterID = 0
iCityRevolutionCivID = 1
iCityRevolutionIndexID = 2
iCityPrevRevIndexID = 3
iCityLocalRevIndexID = 4
iCityRevolutionCounterID = 5

LOG_DEBUG = True
revCultureModifier = 1.0


########################## Event Handling ##########################################################

def registerEvents( customEM, config ) :

        customEM.addEventHandler( 'cityBuilt', onCityBuilt )
        customEM.addEventHandler( 'cityAcquired', onCityAcquired )
        customEM.addEventHandler( 'EndGameTurn', onEndGameTurn )

        LOG_DEBUG = config.getboolean("Revolution", "DebugMode", False)

        revCultureModifier = config.getfloat("Revolution", "RevCultureModifier", 1.0)

def onCityBuilt( argsList ):
        'City Built'
        city = argsList[0]

	initCity( city )

	pPlayer = gc.getPlayer( city.getOwner() )

	if( pPlayer.isBarbarian() ) :
            setCityRevolutionIndex( city, 400 )
            return

	if( not city.area().getID() == pPlayer.getCapitalCity().area().getID() ) :
            setCityRevolutionIndex( city, 350 )
        else :
            setCityRevolutionIndex( city, 250 )

def onCityAcquired( argsList ):
	'City Acquired'

	owner,playerType,city,bConquest,bTrade = argsList

	newOwnerID = city.getOwner()
        newOwner = gc.getPlayer(newOwnerID)
        newOwnerCiv = newOwner.getCivilizationType()
        oldOwnerID = city.getPreviousOwner()
        orgOwnerID = city.getOriginalOwner()		

        if( newOwnerID == gc.getBARBARIAN_PLAYER() and getCityRevolutionCounter(city) > 0 ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - City %s captured by barb rebels!"%(city.getName()))
            oldOwner = gc.getPlayer(oldOwnerID)
            
            if( not oldOwnerID == orgOwnerID ) :
                orgOwner = gc.getPlayer(orgOwnerID)
                #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - City %s originally owned by %s!"%(city.getName(),orgOwner.getCivilizationDescription(0)))

                #if( something ) :
                    # Flip back to original owner

                    # Kill nearby barbs                    

            if( city.countTotalCultureTimes100() > 100*100 ) :
                if( not (oldOwnerID == city.findHighestCulture()) ) :
                    cultOwner = gc.getPlayer( city.findHighestCulture() )
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - City %s culturally dominated by %s!"%(city.getName(),cultOwner.getCivilizationDescription(0)))

                    #if( something ) :
                        # Flip to cultural owner

                        # Gift nearby barbs

            initCity(city)

        elif( newOwnerCiv == getCityRevolutionCiv(city) ) :

            # Did city recently rebel in their favor or currently unhappy?
            if( getCityRevolutionCounter(city) > 0 or (city.unhappyLevel(0) - city.happyLevel()) > 0 ) :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Rebellious city %s is captured by rebel civ %s (%d)!!!"%(city.getName(),newOwner.getCivilizationDescription(0),newOwnerCiv))

                cultPlayer = None
                if( city.countTotalCultureTimes100() > 100*100 ) :
                    if( not city.getPreviousOwner() == city.findHighestCulture() ) :
                        cultPlayer = gc.getPlayer( city.findHighestCulture() )
                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - New rebel city %s culturally dominated by %s"%(city.getName(),cultPlayer.getCivilizationDescription(0)))

                if( city.plot().countTotalCulture() > 100 ) :
                    if( (not city.plot().calculateCulturalOwner() == city.getPreviousOwner()) and cultPlayer == None ) :
                        cultPlayer = gc.getPlayer( city.plot().calculateCulturalOwner() )
                        if( not cultPlayer == None ) :
                            if( cultPlayer.isBarbarian() ) :
                                cultPlayer = None
                            else :
                                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - New rebel city %s is culturally influenced by %s"%(city.getName(),cultPlayer.getCivilizationDescription(0)))

                newCulVal = int( revCultureModifier*max([city.getCulture(oldOwnerID),city.countTotalCultureTimes100()/200]) )
                newPlotVal = int( revCultureModifier*max([city.plot().getCulture(oldOwnerID),city.plot().countTotalCulture()/2]) )
                giveCityCulture( city, newOwnerID, newCulVal, newPlotVal, overwriteHigher = False )

                ix = city.getX()
                iy = city.getY()

                if( newOwner.isHuman() ) :
                    CyInterface().addImmediateMessage("With your capture of %s, support for the rebellion in the city swells ..."%(city.getName()),"")
                elif( city.isVisible( game.getActiveTeam(), False ) ) :
                    CyInterface().addImmediateMessage("The rebel %s have taken control of %s!  Support for the rebellion swells ..."%(newOwner.getCivilizationDescription(0),city.getName()),"")

                [iWorker,iBestDefender,iCounter,iAttack] = getHandoverUnitTypes( city, newOwner )

                if( not iBestDefender == UnitTypes.NO_UNIT ) : newOwner.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                if( not iBestDefender == UnitTypes.NO_UNIT ) : newOwner.initUnit( iBestDefender, ix, iy, UnitAITypes.NO_UNITAI )
                if( not iCounter == UnitTypes.NO_UNIT ) : newOwner.initUnit( iCounter, ix, iy, UnitAITypes.NO_UNITAI )
                newOwner.initUnit( iWorker, ix, iy, UnitAITypes.NO_UNITAI )
                if( gc.getPlayer( newOwnerID ).getNumCities() <= 1 ) :
                    # Extra units for first city captured
                    if( not iAttack == UnitTypes.NO_UNIT ) : newOwner.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )
                    if( not iAttack == UnitTypes.NO_UNIT ) : newOwner.initUnit( iAttack, ix, iy, UnitAITypes.NO_UNITAI )

                # Give a boat to island rebels
                if( city.isCoastal(10) and city.area().getNumCities() < 3 and city.area().getNumTiles() < 15 ) :
                    iAssaultShip = UnitTypes.NO_UNIT
                    for unitID in range(0,gc.getNumUnitInfos()) :
                        if( newOwner.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_SEA ):
                            unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                            # Military unit transport ship
                            if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ASSAULT_SEA ) ):
                                if( unitID == gc.getCivilizationInfo( newOwner.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                                    if( iAssaultShip == UnitTypes.NO_UNIT or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iAssaultShip).getCombat() ) :
                                        iAssaultShip = unitID
                    if( not iAssaultShip == UnitTypes.NO_UNIT ) :
                        newOwner.initUnit( iAssaultShip, ix, iy, UnitAITypes.UNITAI_ASSAULT_SEA )
                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Rebels get a %s to raid motherland"%(PyInfo.UnitInfo(iAssaultShip).getDescription()))

            # City once rebelled as this civ type, but not currently rebellious
            else :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - %s, captured by former rebel identity: %s (%d)!"%(city.getName(),newOwner.getCivilizationDescription(0),newOwnerCiv))
                newCulVal = int( revCultureModifier*max([city.getCulture(oldOwnerID)/2,city.countTotalCultureTimes100()/400]) )
                newPlotVal = int( revCultureModifier*max([city.plot().getCulture(oldOwnerID)/2,city.plot().countTotalCulture()/4]) )
                giveCityCulture( city, newOwnerID, newCulVal, newPlotVal, overwriteHigher = False )
        
            # Init city script data (unit spawn counter, rebel player)
            initCity(city)

        iTurns = city.getOccupationTimer()
        #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Occupations timer is currently %d"%(iTurns))
        setCityRevolutionCounter( city, max([int(1.5*iTurns),3]) )
        setCityWarningCounter( city, 0 )

def onEndGameTurn( argsList ) :

    if( game.getGameTurn()%10 == 0 ) :
        updateAttitudeExtras()

    removeFloatingRebellions()

def updateAttitudeExtras( ) :

    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
        playerI = gc.getPlayer( i )
        for j in range(0,gc.getMAX_CIV_PLAYERS()) :
            playerJ = gc.getPlayer( j )
            attEx = playerI.AI_getAttitudeExtra(j)
            # Odds and rate partially determined by current attitude value
            if( attEx > 0 and game.getSorenRandNum(100,'Rev: Attitude') < attEx*15 ) :
                playerI.AI_changeAttitudeExtra(j, -(1+attEx/10))
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Extra Attitude for %s of %s now %d"%(playerI.getCivilizationDescription(0),playerJ.getCivilizationDescription(0),playerI.AI_getAttitudeExtra(j)))
            elif( attEx < 0 and game.getSorenRandNum(100,'Rev: Attitude') < -attEx*20 ) :
                playerI.AI_changeAttitudeExtra(j, -(attEx/10))
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Extra Attitude for %s of %s now %d"%(playerI.getCivilizationDescription(0),playerJ.getCivilizationDescription(0),playerI.AI_getAttitudeExtra(j)))

def removeFloatingRebellions( ) :
    # Destroy all units for a rebel who is at peace, yet has no cities

    for i in range(0,gc.getMAX_CIV_PLAYERS()) :
        player = gc.getPlayer(i)
        if( not player.isFoundedFirstCity() and player.getNumUnits() > 0 ) :
            if( player.getNumUnits() == player.getNumMilitaryUnits() ) :
                if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Player %d (%s) is a homeless rebel"%(player.getID(),player.getCivilizationDescription(0)))
                if( gc.getTeam( player.getTeam() ).getAtWarCount( True ) == 0 ) :
                    if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Player %d has lost their cause, terminating rebel"%(player.getID()))
                    player.killUnits()

########################## Revolution helper functions ###############################



def computeWarOdds( attacker, victim, area ) :

        warOdds = 0
        if( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_FURIOUS ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Join player furious with owner")
            warOdds += 50
        elif( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_ANNOYED ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Join player annoyed with owner")
            warOdds += 25
        elif( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_PLEASED ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Join player pleased with owner")
            warOdds -= 25
        elif ( attacker.AI_getAttitude(victim.getID()) == AttitudeTypes.ATTITUDE_FRIENDLY ) :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Join player friendly with owner")
            warOdds -= 50
        else :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Join player cautious with owner")
            
        powerFrac = area.getPower(attacker.getID())/(1.0*area.getPower(victim.getID()))
        if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Power fraction is %f for join player"%(powerFrac))
        if( area.getCitiesPerPlayer(attacker.getID()) > 1 and powerFrac > 1.5 ) :
            warOdds += 50
        elif( powerFrac > 1.1 ) :
            warOdds += 25
        elif( powerFrac < .9 ) :
            warOdds -= 25
        elif( area.getCitiesPerPlayer(attacker.getID()) < 1 or powerFrac < .7 ) :
            warOdds -= 50

        if( area.getCitiesPerPlayer(attacker.getID()) > area.getCitiesPerPlayer(victim.getID()) ) :
            warOdds += 10

        iAgg = CvUtil.findInfoTypeNum(gc.getTraitInfo,gc.getNumTraitInfos(),'TRAIT_AGGRESSIVE')
        if( attacker.hasTrait(iAgg) ) :
            warOdds += 10

        return warOdds

def giveTechs( toPlayer, fromPlayer, expensiveVars = [2,3], doTakeAway = True ) :

        # Give all techs known by fromPlayer, except a few of the most expensive
        knownTechs = list()
        mostExpensive = list()
        minMostExpensive = 0
        numMostExpensive = expensiveVars[0] + game.getSorenRandNum(expensiveVars[1],'Rev: Pick num techs')

        fromPlayerTeam = gc.getTeam( fromPlayer.getID() )
        toPlayerTeam = gc.getTeam( toPlayer.getID() )
        
        for techID in range(0,gc.getNumTechInfos()) :
            if( fromPlayerTeam.isHasTech( techID ) ) :
                knownTechs.append( techID )
                if( gc.getTechInfo( techID ).getResearchCost() > minMostExpensive ) :
                    if( len(mostExpensive) < numMostExpensive ) :
                        mostExpensive.append( techID )
                    else :
                        for j in range(0,len(mostExpensive)) :
                            if( gc.getTechInfo( mostExpensive[j] ).getResearchCost() == minMostExpensive ) :
                                mostExpensive.remove( mostExpensive[j] )
                                break
                        mostExpensive.append( techID )
                        minMostExpensive = gc.getTechInfo( mostExpensive[0] ).getResearchCost()
                        for j in range(0,len(mostExpensive)) :
                            if( gc.getTechInfo( mostExpensive[j] ).getResearchCost() < minMostExpensive ) :
                                minMostExpensive = gc.getTechInfo( mostExpensive[j] ).getResearchCost()

            if( doTakeAway and toPlayerTeam.isHasTech(techID) ) :
                # take away all techs
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Taking away %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
                toPlayerTeam.setHasTech(techID,False,toPlayer.getID(),False,False)

        if( doTakeAway ) :
            if( not toPlayer.getTechScore() == 0 ) :
                # Shouldn't have to do this, but tech scores never came out to zero for reincarnated civs ...
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Resetting tech score ...")
                toPlayer.changeTechScore( -toPlayer.getTechScore() )

        for techID in knownTechs :
            if( not techID in mostExpensive ) :
                toPlayerTeam.setHasTech(techID,True,toPlayer.getID(),False,False)
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Revolt - Giving rev %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
            else :
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - NOT giving rev %s"%(PyInfo.TechnologyInfo(techID).getDescription()))
                

def giveCityCulture( pCity, iPlayer, newCityVal, newPlotVal, overwriteHigher = False ) :
        # Places this culture value in city and city plot
        # Places half this value in neighboring plots

        if( iPlayer < 0 or iPlayer >= gc.getBARBARIAN_PLAYER() ) :
            return

        if( overwriteHigher or newCityVal > pCity.getCulture(iPlayer) ) :
            pCity.setCulture( iPlayer, newCityVal, True )
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Culture set to %d"%(pCity.getCulture(iPlayer)))
            
            if( overwriteHigher or newPlotVal > pCity.plot().getCulture(iPlayer) ) :
                pCity.plot().setCulture( iPlayer, newPlotVal, True )

            gameMap = gc.getMap()

            for ix in range(pCity.getX()-1,pCity.getX()+2) :
                for iy in range(pCity.getY()-1,pCity.getY()+2) :

                    if( ix < 0 ) :
                        if( gameMap.isWrapX() ) :
                            ix = CyMap().getGridWidth() + ix
                        else :
                            continue
                    elif( ix >= CyMap().getGridWidth() ) :
                        if( gameMap.isWrapX() ) :
                            ix = ix - CyMap().getGridWidth()
                        else :
                            continue

                    if( iy < 0 ) :
                        if( gameMap.isWrapY() ) :
                            iy = CyMap().getGridHeight() + iy
                        else :
                            continue
                    elif( iy >= CyMap().getGridHeight() ) :
                        if( gameMap.isWrapY() ) :
                            iy = iy - CyMap().getGridHeight()
                        else :
                            continue

                    pPlot = gc.getMap().plot(ix,iy)

                    if( pPlot.countTotalCulture() > 0 ) :
                        if( overwriteHigher or newPlotVal/2 > pPlot.getCulture(iPlayer) ) :
                            pPlot.setCulture( iPlayer, newPlotVal/2, True )
                    
        else :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %s already has a lot of culture here"%(gc.getPlayer(iPlayer).getCivilizationDescription(0)))

                                             
def getHandoverUnitTypes( city, pPlayer ) :

        workerClass = CvUtil.findInfoTypeNum(gc.getUnitClassInfo,gc.getNumUnitClassInfos(),'UNITCLASS_WORKER')
        iWorker = gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(workerClass)
        iBestDefender = UnitTypes.NO_UNIT
        iCounter = UnitTypes.NO_UNIT
        iAttack = UnitTypes.NO_UNIT
        for unitID in range(0,gc.getNumUnitInfos()) :
            if( city.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_LAND ): # canTrain only does technology check
                unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                # Defender (Archer,Longbow)
                if( gc.getUnitInfo(unitID).getDefaultUnitAIType() == UnitAITypes.UNITAI_CITY_DEFENSE ):
                    if( (iBestDefender == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iBestDefender).getCombat() ) :
                        #if( LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ def units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                        if( unitID == gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                            iBestDefender = unitID
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev - Best defender set to %s"%(PyInfo.UnitInfo(iBestDefender).getDescription()))
                # Counter (Axemen,Phalanx)
                if( gc.getUnitInfo(unitID).getUnitAIType(UnitAITypes.UNITAI_COUNTER) ):
                    if( (iCounter == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() >= gc.getUnitInfo(iCounter).getCombat() ) :
                        #if( LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ counter units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                        if( unitID == gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                            iCounter = unitID
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev  - Best counter unit set to %s"%(PyInfo.UnitInfo(iCounter).getDescription()))
                # Assault units
                if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ATTACK ) ):
                    if( (iAttack == UnitTypes.NO_UNIT) or gc.getUnitInfo(unitID).getCombat() > gc.getUnitInfo(iAttack).getCombat() ) :
                        #if( LOG_DEBUG ) : CvUtil.pyPrint("  BC - Civ attack units return value: %d from %d, comp to %d"%(gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass),unitClass,unitID))
                        if( unitID == gc.getCivilizationInfo( pPlayer.getCivilizationType() ).getCivilizationUnits(unitClass) ) :
                            iAttack = unitID
                            #if( LOG_DEBUG ) : CvUtil.pyPrint("Rev  - Best attack set to %s"%(PyInfo.UnitInfo(iAttack).getDescription()))

        return [iWorker,iBestDefender,iCounter,iAttack]

def getUprisingUnitTypes( pCity, pRevPlayer, isJoinWar ) :

        spawnableUnits = list()
        trainableUnits = list()

        owner = gc.getPlayer( pCity.getOwner() )
        ownerPy = PyPlayer( pCity.getOwner() )
        enemyPy = None
        if( isJoinWar ) :
            enemyPy = PyPlayer( pRevPlayer.getID() )

        bIsBarb = (pRevPlayer.getID() == gc.getBARBARIAN_PLAYER())

        for unitID in range(0,gc.getNumUnitInfos()) :
            ownerUnits = ownerPy.getUnitsOfType( unitID )
            iOwnerEra = owner.getCurrentEra()
            unitInfo = gc.getUnitInfo(unitID)
            if( not unitInfo.getPrereqAndTech() == TechTypes.NO_TECH ) :
                unitTechInfo = gc.getTechInfo( unitInfo.getPrereqAndTech() )

                if( unitTechInfo.getEra() > iOwnerEra - 2 ) :
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel: %s requires knowledge of %s"%(unitInfo.getDescription(),unitTechInfo.getDescription()))
                    if( len( ownerUnits ) > 0 ) :
                        if( ownerUnits[0].canAttack() ) :
                            if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Owner has %s"%(PyInfo.UnitInfo(unitID).getDescription()))
                            if( unitInfo.getDomainType() == DomainTypes.DOMAIN_LAND and ( unitInfo.getUnitAIType(UnitAITypes.UNITAI_ATTACK) or unitInfo.getUnitAIType(UnitAITypes.UNITAI_COUNTER) )):
                                unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                                for unit in ownerUnits :
                                    if( plotDistance( unit.getX(), unit.getY(), pCity.getX(), pCity.getY() ) < 5 ) :
                                        if( bIsBarb ) :
                                            spawnUnitID = gc.getCivilizationInfo( owner.getCivilizationType() ).getCivilizationUnits(unitClass)
                                        else :
                                            spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                        spawnableUnits.append( spawnUnitID )
                                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from owner %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))
                                        if( spawnableUnits.count( spawnUnitID ) > 7 ) :
                                            break

                                for unit in ownerUnits :
                                    if( unit.area().getID() == pCity.area().getID() ):
                                        if( bIsBarb ) :
                                            spawnUnitID = gc.getCivilizationInfo( owner.getCivilizationType() ).getCivilizationUnits(unitClass)
                                        else :
                                            spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                        spawnableUnits.append( spawnUnitID )
                                        if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from Area %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))
                                        break

                    if( not enemyPy == None ) :
                        enemyUnits = enemyPy.getUnitsOfType( unitID )
                        if( len( enemyUnits ) > 0 ) :
                            if( enemyUnits[0].canAttack() ) :
                                if( unitInfo.getDomainType() == DomainTypes.DOMAIN_LAND and unitInfo.getUnitAIType( UnitAITypes.UNITAI_ATTACK )  ):
                                    unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                                    for unit in enemyUnits :
                                        if( plotDistance( unit.getX(), unit.getY(), pCity.getX(), pCity.getY() ) < 7 ) :
                                            if( bIsBarb ) :
                                                # Should never get here
                                                spawnUnitID = unitID
                                            else :
                                                spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                                            spawnableUnits.append( spawnUnitID )
                                            if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can spawn from enemy %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))

            
            if( pCity.canTrain(unitID,False,False) and gc.getUnitInfo(unitID).getDomainType() == DomainTypes.DOMAIN_LAND ):
                if( gc.getUnitInfo(unitID).getUnitAIType( UnitAITypes.UNITAI_ATTACK ) ):
                    unitClass = gc.getUnitInfo(unitID).getUnitClassType()
                    if( bIsBarb ) :
                        spawnUnitID = gc.getCivilizationInfo( owner.getCivilizationType() ).getCivilizationUnits(unitClass)
                    else :
                        spawnUnitID = gc.getCivilizationInfo( pRevPlayer.getCivilizationType() ).getCivilizationUnits(unitClass)
                        
                    if( not spawnUnitID in trainableUnits ) :
                        trainableUnits.append( spawnUnitID )
                        if( gc.getUnitInfo(unitID).getCombat() > 4 ) :
                            trainableUnits.append( spawnUnitID )
                            if( gc.getUnitInfo(unitID).getCombat() > 15 ) :
                                trainableUnits.append( spawnUnitID )
                        #if( LOG_DEBUG ) : CvUtil.pyPrint("Rebel:  Can build %s"%(PyInfo.UnitInfo(spawnUnitID).getDescription()))

        if( len(spawnableUnits) < 1 ) :
            spawnableUnits = trainableUnits

        return spawnableUnits

def getSpawnablePlots( iPlotX, iPlotY, pSpawnPlayer, bLand = True, bIncludePlot = True, bIncludeCities = False, bSameArea = True, iRange = 2, iSpawnPlotOwner = -1, bCheckForEnemy = True ) :

        spawnablePlots = list()

        gameMap = gc.getMap()

        try :
            iBaseArea = gameMap.plot(iPlotX,iPlotY).area().getID()
        except AttributeError :
            if( bSameArea ) : print "WARNING: Passed an arealess plot!"
            iBaseArea = -1
            bSameArea = False
        iBasePlotOwner = gameMap.plot(iPlotX,iPlotY).getOwner()
        iNumPlotsChecked = 0

        for ix in range(iPlotX-iRange,iPlotX+iRange+1) :
            for iy in range(iPlotY-iRange,iPlotY+iRange+1) :

                if( ix < 0 ) :
                    if( gameMap.isWrapX() ) :
                        ix = CyMap().getGridWidth() + ix
                    else :
                        continue
                elif( ix >= CyMap().getGridWidth() ) :
                    if( gameMap.isWrapX() ) :
                        ix = ix - CyMap().getGridWidth()
                    else :
                        continue

                if( iy < 0 ) :
                    if( gameMap.isWrapY() ) :
                        iy = CyMap().getGridHeight() + iy
                    else :
                        continue
                elif( iy >= CyMap().getGridHeight() ) :
                    if( gameMap.isWrapY() ) :
                        iy = iy - CyMap().getGridHeight()
                    else :
                        continue

                if( not bIncludePlot and ix == iPlotX and iy == iPlotY ) :
                    continue

                pPlot = gameMap.plot( ix, iy )

                if( pPlot.isImpassable() ):
                    continue

                iNumPlotsChecked += 1

                if( bLand and pPlot.isWater() ) :
                    continue

                if( not bLand and not pPlot.isWater() ) :
                    continue

                if( not bIncludeCities and pPlot.isCity() ) :
                    continue

                if( bSameArea and not iBaseArea == pPlot.area().getID() ) :
                    continue

                if( bCheckForEnemy ) :
                    pSpawnTeam = gc.getTeam( pSpawnPlayer.getTeam() )
                    for i in range(0,pPlot.getNumUnits()) :
                        pUnit = pPlot.getUnit(i)
                        if( pSpawnTeam.isAtWar(pUnit.getTeam()) ) :
                            continue

                # When iSpawnPlotOwner >= 0, plot owner must be either iSpawnPlotOwner, iBasePlotOwner, or no one
                if( iSpawnPlotOwner < 0 or pPlot.getOwner() == iSpawnPlotOwner or pPlot.getOwner() == iBasePlotOwner or pPlot.getOwner() == PlayerTypes.NO_PLAYER ) :
                    spawnablePlots.append( [ix,iy] )

        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Found %d plots out of %d checked"%(len(spawnablePlots),iNumPlotsChecked))

##        for plot in spawnablePlots :
##            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - SP: %d, %d"%(plot[0],plot[1]))

        return spawnablePlots

def getEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer ) :

        pEnemyOfTeam = gc.getTeam( gc.getPlayer(iEnemyOfPlayer).getTeam() )
        gameMap = gc.getMap()
        pPlot = gameMap.plot(iPlotX,iPlotY)

        enemyUnits = list()

        for i in range(0,pPlot.getNumUnits()) :
            pUnit = pPlot.getUnit(i)
            pUnitTeam = gc.getTeam( pUnit.getTeam() )

            enemyUnits.append( pUnit )
            #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Unit %s, id %d, player %d, at %d,%d"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner(),pUnit.plot().getX(),pUnit.plot().getY()))

        return enemyUnits

def moveEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer, iMoveToX, iMoveToY, iInjureMax = 0, bDestroyNonLand = True ) :

        unitList = getEnemyUnits( iPlotX, iPlotY, iEnemyOfPlayer )

        for pUnit in unitList :               
                if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
                unitPlot = pUnit.plot()
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - from %d, %d"%(unitPlot.getX(),unitPlot.getY()))

        if( iInjureMax > 0 ) :
            for pUnit in unitList :
                iInjure = iInjureMax/2 + game.getSorenRandNum(iInjureMax/2,'Rev: Wound retreating units')
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %d starts at %d"%(pUnit.getID(),pUnit.getDamage()))
                if( pUnit.canFight() and pUnit.getDamage() < iInjure ) :
                    pUnit.setDamage( iInjure, iEnemyOfPlayer )
                    #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %d injured to %d"%(pUnit.getID(),pUnit.getDamage()))

        pPlot = gc.getMap().plot(iMoveToX,iMoveToY)

        toKillList = list()
        for pUnit in unitList :
            if( not pUnit.getDomainType() == DomainTypes.DOMAIN_LAND or not pUnit.canMoveInto(pPlot,False,False,True) ) :
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Pulling out %s for subsequent kill"%(pUnit.getName()))
                toKillList.append(pUnit)
            else :
                
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Moving %s (id: %d, player: %d)"%(pUnit.getName(),pUnit.getID(),pUnit.getOwner()))
                #unitPlot = pUnit.plot()
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - from %d, %d"%(unitPlot.getX(),unitPlot.getY()))
                pUnit.setXY( iMoveToX, iMoveToY )
                #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - to %d, %d"%(pUnit.plot().getX(),pUnit.plot().getY()))

        #if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Units moved")

        for pUnit in toKillList :
            if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - Killing %s"%(pUnit.getName()))
            pUnit.kill(False,iEnemyOfPlayer)



def getRevolutionCityLists( pPlayer ) :

    playerPy = PyPlayer( pPlayer.getID() )
    cityList = playerPy.getCityList()

    pTeam = gc.getTeam( pPlayer.getTeam() )

    revReadyCities = list()
    revInstigatorCities = list()
    warnCities = list()
    unhappyCities = list()

    capRevIdx = 0

    for city in cityList :
        pCity = city.GetCy()

        revIdx = getCityRevolutionIndex( pCity )
        prevRevIdx = getCityPrevRevIndex( pCity )
        localRevIdx = getCityLocalRevIndex( pCity )

        numUnhappy = pCity.angryPopulation(0)
        modifier = pCity.getPopulation()*pCity.getWarWearinessPercentAnger()/100
        modNumUnhappy = pCity.angryPopluation( -modifier )

        if( LOG_DEBUG ) : CvUtil.pyPrint("  Rev - %s has numUnhappy %d, mod %d, modNumUn %d"%(pCity.getName(),numUnhappy,modifier,modNumUnhappy))
        
        #if( modNumUnhappy > 0 ) :
            

def getModNumUnhappy( pCity, wwMod = 2.0 ) :

    modifier = int( wwMod*pCity.getPopulation()*pCity.getWarWearinessPercentAnger()/1000 )

    modNumUnhappy = pCity.angryPopulation( -modifier )

    if( LOG_DEBUG and pCity.angryPopulation(0) > 0 and game.getGameTurn()%25 == 0 ) :
        CvUtil.pyPrint("  Rev - %s has numUnhappy %d, mod %d, modNumUn %d"%(pCity.getName(),pCity.angryPopulation(0),modifier,modNumUnhappy))

    if( modNumUnhappy <= 0 ) :
        numHappy = max( [0, pCity.happyLevel() - pCity.unhappyLevel(0)] )
        numHappy = min( [numHappy, pCity.getPopulation()] )
        return -numHappy
    else :
        return modNumUnhappy

########################## City Script Data Functions ##############################################

def initCity(pCity):        
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            if( len(szScriptData) > iCityRevolutionCounterID + 1 ) :
                # Something in there already
                szScriptData[iCityWarningCounterID] = 0
                szScriptData[iCityRevolutionCivID] = -1
                szScriptData[iCityRevolutionIndexID] = 0
                szScriptData[iCityPrevRevIndexID] = 0
                szScriptData[iCityLocalRevIndexID] = 0
                szScriptData[iCityRevolutionCounterID] = 0
            else :
                szScriptData = [0, -1, 0, 0, 0, 0]
        except (EOFError, IndexError,MemoryError):
            szScriptData = [0, -1, 0, 0, 0, 0]
        
        # Save Script Data - CityRevolutionIndex
        pCity.setScriptData(pickle.dumps(szScriptData))

# ----------- Warning Counter functions ------------------

def getCityWarningCounter(pCity):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityWarningCounter
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityWarningCounter = szScriptData[iCityWarningCounterID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityWarningCounter = szScriptData[iCityWarningCounterID]

        #print("  Revolt - Loading CityWarningCounter, val: %d"%(iCityWarningCounter))
        
        return iCityWarningCounter

def setCityWarningCounter(pCity, iValue ) :
        
        # Load Script Data - CityWarningCounter
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityWarningCounterID] = iValue
        except (EOFError, IndexError):
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityWarningCounterID] = iValue
        
        # Save Script Data - CityWarningCounter
        pCity.setScriptData(pickle.dumps(szScriptData))

def changeCityWarningCounter(pCity, iChange):
		      
        # Load Script Data - CityWarningCounter
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iWarningCounter = szScriptData[iCityWarningCounterID]
        except (EOFError, IndexError):
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iWarningCounter = szScriptData[iCityWarningCounterID]
        
        iWarningCounter += iChange
        
        # Save Script Data - CityWarningCounter
        szScriptData[iCityWarningCounterID] = iWarningCounter
        pCity.setScriptData(pickle.dumps(szScriptData))

# ----------- Revolution player functions ------------------

def getCityRevolutionCiv(pCity):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityRevolutionCiv
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionCiv = szScriptData[iCityRevolutionCivID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionCiv = szScriptData[iCityRevolutionCivID]

        #print("  Revolt - Loading CityRevolutionCiv, val: %d"%(iCityRevolutionCiv))
        
        return iCityRevolutionCiv
        
def setCityRevolutionCiv(pCity, iValue):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityRevolutionCiv
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityRevolutionCivID] = iValue
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityRevolutionCivID] = iValue
        
        # Save Script Data - CityRevolutionCiv
        pCity.setScriptData(pickle.dumps(szScriptData))

# ----------- Revolution index functions ------------------

def getCityRevolutionIndex(pCity):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityRevolutionIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionIndex = szScriptData[iCityRevolutionIndexID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionIndex = szScriptData[iCityRevolutionIndexID]

        #print("  Revolt - Loading CityRevolutionIndex, val: %d"%(iCityRevolutionIndex))
        
        return iCityRevolutionIndex
        
def setCityRevolutionIndex(pCity, iValue):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityRevolutionIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityRevolutionIndexID] = iValue
        except (EOFError, IndexError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityRevolutionIndexID] = iValue
        
        # Save Script Data - CityRevolutionIndex
        pCity.setScriptData(pickle.dumps(szScriptData))
        
def changeCityRevolutionIndex(pCity, iChange):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() )
        
        # Load Script Data - CityRevolutionIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionIndex = szScriptData[iCityRevolutionIndexID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionIndex = szScriptData[iCityRevolutionIndexID]
        
        iCityRevolutionIndex += iChange
        
        # Save Script Data - CityRevolutionIndex
        szScriptData[iCityRevolutionIndexID] = iCityRevolutionIndex
        pCity.setScriptData(pickle.dumps(szScriptData))

# ----------- Revolution previous index functions ------------------

def getCityPrevRevIndex(pCity):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityPrevRevIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityPrevRevIndex = szScriptData[iCityPrevRevIndexID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityPrevRevIndex = szScriptData[iCityPrevRevIndexID]

        #print("  Revolt - Loading CityPrevRevIndex, val: %d"%(iCityPrevRevIndex))
        
        return iCityPrevRevIndex
        
def setCityPrevRevIndex(pCity, iValue):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityPrevRevIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityPrevRevIndexID] = iValue
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityPrevRevIndexID] = iValue
        
        # Save Script Data - CityPrevRevIndex
        pCity.setScriptData(pickle.dumps(szScriptData))

# ----------- Revolution local index functions ------------------

def getCityLocalRevIndex(pCity):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityLocalRevIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityLocalRevIndex = szScriptData[iCityLocalRevIndexID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityLocalRevIndex = szScriptData[iCityLocalRevIndexID]

        #print("  Revolt - Loading CityLocalRevIndex, val: %d"%(iCityLocalRevIndex))
        
        return iCityLocalRevIndex
        
def setCityLocalRevIndex(pCity, iValue):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityLocalRevIndex
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityLocalRevIndexID] = iValue
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityLocalRevIndexID] = iValue
        
        # Save Script Data - CityLocalRevIndex
        pCity.setScriptData(pickle.dumps(szScriptData))

# ----------- Revolution Counter functions ------------------

def getCityRevolutionCounter(pCity):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityRevolutionCounter
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionCounter = szScriptData[iCityRevolutionCounterID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionCounter = szScriptData[iCityRevolutionCounterID]

        #print("  Revolt - Loading CityRevolutionCounter, val: %d"%(iCityRevolutionCounter))
        
        return iCityRevolutionCounter
        
def setCityRevolutionCounter(pCity, iValue):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() ) 
        
        # Load Script Data - CityRevolutionCounter
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityRevolutionCounterID] = iValue
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            szScriptData[iCityRevolutionCounterID] = iValue
        
        # Save Script Data - CityRevolutionCounter
        pCity.setScriptData(pickle.dumps(szScriptData))
        
def changeCityRevolutionCounter(pCity, iChange):
        
        #pPlayer = gc.getPlayer( pCity.getOwner() )
        
        # Load Script Data - CityRevolutionCounter
        try :
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionCounter = szScriptData[iCityRevolutionCounterID]
        except (EOFError, IndexError,MemoryError):
            #print "  Rev - Script values for %s not properly initialized"%(pCity.getName())
            initCity(pCity)
            szScriptData = pickle.loads(pCity.getScriptData())
            iCityRevolutionCounter = szScriptData[iCityRevolutionCounterID]
        
        iCityRevolutionCounter += iChange
        
        # Save Script Data - CityRevolutionCounter
        szScriptData[iCityRevolutionCounterID] = iCityRevolutionCounter
        pCity.setScriptData(pickle.dumps(szScriptData))
        


########################## Player modification functions ###########################################

def changeCiv( playerIdx, newCivType, newLeaderType, teamIdx = -1 ) :
    # Changes specified players civ, leader and/or team
    # Does not change isHuman setting

    player = gc.getPlayer(playerIdx)
    print 'Changing player ... is human? %d'%(player.isHuman() or (gc.getActivePlayer().getID() == player.getID()))
    success = game.changePlayer( playerIdx, newCivType, newLeaderType, teamIdx, (player.isHuman() or (gc.getActivePlayer().getID() == playerIdx)), True )
    print 'After change ... is human? %d'%(player.isHuman() or (gc.getActivePlayer().getID() == player.getID()))
    #game.convertUnits( playerIdx )

    return success

def changeHuman( newHumanIdx, oldHumanIdx ) :

    newPlayer = gc.getPlayer(newHumanIdx)
    oldPlayer = gc.getPlayer(oldHumanIdx)

    game.setActivePlayer( newHumanIdx, False )
    success = game.changePlayer( newHumanIdx, newPlayer.getCivilizationType(), newPlayer.getLeaderType(), -1, True, False )
    if( not success ) :
        CyInterface().addImmediateMessage("ERROR: Failure to set human flag in new player.  You will probably need to reload a save.","")
    success = game.changePlayer( oldHumanIdx, oldPlayer.getCivilizationType(), oldPlayer.getLeaderType(), -1, False, False )
    if( not success ) :
        CyInterface().addImmediateMessage("ERROR: Failure to unset human flag in old player.  You will probably need to reload a save.","")

    return success
